package cn.fw.morax.common.utils;


import cn.fw.common.data.entity.BaseEntity;
import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.page.BasePageQuery;
import cn.fw.common.page.PageQuery;
import cn.fw.morax.common.constant.Constant;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.api.R;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.BeanUtils;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.time.ZoneId;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * Date: 2022/4/3
 * Description: 通用工具类
 */
public class PublicUtil {

    /**
     * uuid生成器
     *
     * @return
     */
    public static String nextId() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    /**
     * uuid生成器
     *
     * @return
     */
    public static String getUUID() {
        String uuid = UUID.randomUUID().toString().replace("-", "").substring(0, 16);
        Random random = new Random();
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0, length = uuid.length(); i < length; i++) {
            char c = uuid.charAt(i);
            if (Character.isLowerCase(c) && random.nextBoolean()) {
                stringBuilder.append(Character.toUpperCase(c));
            } else {
                stringBuilder.append(c);
            }
        }
        return stringBuilder.toString();
    }

    /**
     * 非空判断
     *
     * @param obj
     * @return
     */
    public static boolean isNotEmpty(Object obj) {
        return !isEmpty(obj);
    }

    /**
     * 为空判断
     *
     * @param obj
     * @return
     */
    public static boolean isEmpty(Object obj) {
        if (obj == null) {
            return true;
        } else if (obj instanceof String) {
            return "".equals(obj);
        } else if (obj instanceof Collection) {
            Collection map1 = (Collection) obj;
            return map1.size() == 0;
        } else if (obj instanceof Map) {
            Map map = (Map) obj;
            return map.size() == 0;
        } else {
            return obj.getClass().isArray() && Array.getLength(obj) == 0;
        }
    }

    /**
     * 转换分页
     *
     * @param page
     * @param mapper
     * @param <E>
     * @param <R>
     * @return
     */
    public static <E, R> AppPage<R> toPage(PageData<E> page, Function<? super E, ? extends R> mapper) {
        AppPage result = page.toAppPage();
        result.setData(Optional.ofNullable(page.getRecords())
                .orElseGet(() -> new ArrayList<>())
                .stream()
                .map(mapper)
                .collect(Collectors.toList()));
        return result;
    }

    /**
     * 分页查询
     *
     * @param pageQuery
     * @param totalSize
     * @param mapper
     * @param <E>
     * @return
     */
    public static <E, T extends BasePageQuery> AppPage<E> queryPage(T pageQuery, Long totalSize, Function<T, List<E>> mapper) {
        if (totalSize <= 0L) {
            return AppPage.empty(pageQuery);
        }
        Integer pageSize = pageQuery.getPageSize();
        Integer current = pageQuery.getCurrent();
        Integer startIndex ;
        if (current.equals(0)) {
            startIndex = 0;
            current = 1;
        } else {
            startIndex = (current - 1) * pageSize;
        }
        //current(页码)作为 偏移量
        pageQuery.setCurrent(startIndex);
        pageQuery.setPageSize(pageSize);
        AppPage<E> page = AppPage.empty(pageQuery);
        page.setTotal(totalSize);
        page.setData(mapper.apply(pageQuery));
        page.setCurrent(current);
        return page;
    }

    /**
     * cglib对象复制
     *
     * @param source           拷贝数据源
     * @param targetClass      拷贝目标类
     * @param ignoreProperties 忽略属性
     * @param <E>              数据源类泛型
     * @param <R>              目标类泛型
     * @return
     */
    public static <E, R> R copy(E source, Class<R> targetClass, String... ignoreProperties) {
        R target = null;
        if (isNotEmpty(source)) {
            try {
                target = targetClass.newInstance();
                BeanUtils.copyProperties(source, target, ignoreProperties);
                copyBaseEntityId(source, targetClass, target);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return target;
    }

    /**
     * 拷贝基础类的id属性
     * @param source
     * @param targetClass
     * @param target
     * @param <E>
     * @param <R>
     */
    public static <E, R> void copyBaseEntityId(E source, Class<R> targetClass, R target) {
        //非基础实体类不进行拷贝
        if (! (source instanceof BaseEntity)) {
            return;
        }
        Class<?> sourceClass = source.getClass();
        try {
            Method sourceMethod = sourceClass.getMethod("getId");
            Method targetMethod = targetClass.getMethod("setId", Long.class);
            Object obj = sourceMethod.invoke(source);
            targetMethod.invoke(target, obj);
        } catch (Exception e) {
        }
    }

    /**
     * cglib对象复制
     *
     * @param sourceList       拷贝数据源列表
     * @param targetClass      拷贝目标类
     * @param ignoreProperties 忽略属性
     * @param <E>              数据源类泛型
     * @param <R>              目标类泛型
     * @param <R>              目标类泛型
     * @return
     */
    public static <E, R> List<R> copyList(List<E> sourceList, Class<R> targetClass, String... ignoreProperties) {
        if (isEmpty(sourceList)) {
            return new ArrayList<>();
        }
        return Optional.ofNullable(sourceList)
                .orElseGet(() -> new ArrayList<>())
                .stream()
                .map(e -> copy(e, targetClass, ignoreProperties))
                .collect(Collectors.toList());
    }

    /**
     * 获取前一天
     * @param localDate
     * @return
     */
    public static LocalDate getPreviousDay(LocalDate localDate) {
        return localDate.plusDays(-1);
    }

    /**
     * 获取最后一天
     * @param localDate
     * @return
     */
    public static LocalDate getEndDayOfMonth(LocalDate localDate) {
        return localDate.with(TemporalAdjusters.lastDayOfMonth());
    }

    /**
     * 获取最后一天
     * @param localDate
     * @return
     */
    public static Date getExpTimeOfMonth(LocalDate localDate) {
        LocalDate expTime = localDate.with(TemporalAdjusters.lastDayOfMonth());
        return Date.from(expTime.atTime(23, 59, 59).atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * 获取正式员工开始计算绩效的时间
     * @param regularTime   转正时间
     * @return
     */
    public static LocalDate getKpiDateOfRegularStaff(LocalDate regularTime) {
        LocalDate startKpiMonth = regularTime.plusMonths(3);
        return LocalDate.from(startKpiMonth).with(TemporalAdjusters.firstDayOfMonth());
    }

    /**
     * 获取试用期员工开始计算绩效的时间
     * @param entryDate   入职时间
     * @return
     */
    public static LocalDate getKpiDateOfProbationStaff(LocalDate entryDate) {
        LocalDate startKpiMonth = entryDate.plusMonths(6);
        return LocalDate.from(startKpiMonth).with(TemporalAdjusters.firstDayOfMonth());
    }

    /**
     * 获取调岗员工开始计算绩效的时间
     * @param gainPostTime  获得新岗位时间
     * @return
     */
    public static LocalDate getKpiDateOfTransferPostStaff(LocalDate gainPostTime) {
        LocalDate startKpiMonth = gainPostTime.plusMonths(3);
        return LocalDate.from(startKpiMonth).with(TemporalAdjusters.firstDayOfMonth());
    }

    /**
     * 获取待岗员工开始计算绩效的时间
     * @param awaitJobEndDate   待岗结束时间
     * @return
     */
    public static LocalDate getKpiDateOfAwaitJobStaff(LocalDate awaitJobEndDate) {
        LocalDate startKpiMonth = awaitJobEndDate.plusMonths(3);
        return LocalDate.from(startKpiMonth).with(TemporalAdjusters.firstDayOfMonth());
    }

    /**
     * 获取当月第一天
     * @return
     */
    public static LocalDate getCurMonthFirstDay() {
        return LocalDate.now().with(TemporalAdjusters.firstDayOfMonth());
    }

    /**
     * 获取下个月第一天
     * @return
     */
    public static LocalDate getNextMonthFirstDay() {
        return getCurMonthFirstDay().plusMonths(1);
    }

    /**
     * 获取当月最后一天
     * @return
     */
    public static LocalDate getCurMonthEndDay() {
        return LocalDate.now().with(TemporalAdjusters.lastDayOfMonth());
    }

    /**
     * 获取昨日或最后一天
     * @return
     */
    public static LocalDate getYesterdayOrEndOfMonthDay(YearMonth yearMonth) {
        if (YearMonth.now().equals(yearMonth)) {
            return LocalDate.now().minusDays(1);
        }
        return yearMonth.atEndOfMonth();
    }

    /**
     * 为空断言处理
     *
     * @param obj
     * @param message
     */
    public static void assertEmpty(Object obj, String message) {
        if (isNotEmpty(obj)) {
            throw new BusinessException(message);
        }
    }

    public static void main(String[] args) {
        System.out.println(nextId());
        System.out.println(getUUID());
        System.out.println(getUUID());
        System.out.println(getUUID());

        System.out.println(getYesterdayOrEndOfMonthDay(YearMonth.now()));
        System.out.println(getYesterdayOrEndOfMonthDay(YearMonth.now().minusMonths(1)));
    }

}

