package cn.fw.morax.domain.db.kpi;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.common.data.mybatis.handler.IntegerListTypeHandler;
import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.morax.domain.enums.ApplicableRoleEnum;
import cn.fw.morax.domain.enums.DataScopeTypeEnum;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.*;

import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-2
 * @desc : 指标库
 */
@Data
@TableName(autoResultMap = true)
@AllArgsConstructor
@NoArgsConstructor
@Builder
@EqualsAndHashCode(callSuper = true)
public class Indicators extends BaseAuditableTimeEntity<Indicators, Long> {
    /**
     * 指标编号
     */
    private String indicatorCode;
    /**
     * 指标名称
     */
    private String indicatorName;
    /**
     * 归属系统id
     */
    private Long sysId;
    /**
     * 归属系统名称
     */
    private String sysName;
    /**
     * 归属系统前缀
     */
    private String sysPrefix;
    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    private DataTypeEnum dataType;
    /**
     * 适用角色类型;1: 全部角色 2:全部管理角色 3:自定义
     */
    private ApplicableRoleEnum roleType;
    /**
     * 适用角色编码
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> roleCodes;
    /**
     * 适用角色名称
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> roleNames;
    /**
     * 启用/禁用;
     * 0:禁用 1:启用
     */
    private Boolean enable;
    /**
     * 内部指标
     * 0:否 1:是
     */
    private Boolean innerIndicator;
    /**
     * 逻辑删除
     */
    private Boolean yn;
    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额
     */
    private TargetTypeEnum targetType;

    /**
     * 适用范围类型; 1. 绩效组, 2.薪酬组, 3. 考评组
     */
    @TableField(typeHandler = IntegerListTypeHandler.class)
    private List<Integer> applyTypes;

    /**
     * 数据范围类型; 1. 人员, 2.门店
     */
    @TableField(typeHandler = IntegerListTypeHandler.class)
    private List<Integer> dataScopeTypes;

    /**
     * 报表路径
     */
    private String reportPath;

}