package cn.fw.morax.domain.dto.eval;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.morax.domain.enums.EvalScopeEnum;
import cn.fw.morax.domain.enums.EvalTypeEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.Future;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.time.LocalDate;
import java.util.List;

/**
 * <p>
 * 考评组
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalGroupDTO {

    private static final long serialVersionUID = 1L;
    /**
     * 考评组id（前一次）
     */
    private Long id;

    /**
     * 考评组编码
     */
    private String egc;

    /**
     * 考评，活动名称
     */
    @NotBlank(message = "名称不能为空")
    @Length(max = 255, message = "名称超出最大限制")
    private String name;

    /**
     * 考评范围; 1:门店考评 2:人员考评
     */
    @NotNull(message = "考评范围不能为空")
    private EvalScopeEnum scopeType;

    /**
     * 门店奖惩分配角色编码
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> roleCodes;
    /**
     * 门店奖惩分配角色名称
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> roleNames;

//    /**
//     * 考评类型; 1:活动考评 2:其他考评
//     */
//    @NotNull(message = "考评类型不能为空")
//    private EvalTypeEnum type;

    /**
     * 岗位id
     */
//    @NotNull(message = "岗位id不能为空")
    private Long postId;

    /**
     * 岗位名称
     */
//    @NotBlank(message = "岗位名称不能为空")
    private String postName;

    /**
     * 适用门店ids
     */
    @NotNull(message = "适用门店id不能为空")
    @Size(min = 1, message = "适用门店id不能为空")
    private List<Long> shopIds;

    /**
     * 适用门店名称
     */
    @NotNull(message = "适用门店名称不能为空")
    @Size(min = 1, message = "适用门店名称不能为空")
    private List<String> shopNames;

    /**
     * 集团id
     force     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_NAME)
    private String userName;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_ID)
    private Long userId;

    /**
     * 生效时间
     */
//    @Future(message = "生效时间不能在当前时间之前")
    @NotNull(message = "生效时间不能为空")
    private LocalDate beginTime;

    /**
     * 结束时间
     */
    @NotNull(message = "结束时间不能为空")
    @Future(message = "结束时间不能在当前时间之前")
    private LocalDate overTime;

    /**
     * 考评指标
     */
    @Valid
    private List<EvalGroupIndicatorDTO> indicators;

    /**
     * 考评奖惩
     */
    @Valid
    private List<EvalGroupRewardDTO> rewards;

//    /**
//     * 状态;1:审批中 2:审批拒绝 3:生效中 4:草稿
//     */
//    private Integer status;
//
//    /**
//     * 包含指标数量
//     */
//    private Integer indicatorNum;

}
