package cn.fw.morax.domain.dto.eval;

import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.enums.TargetCalcTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 考评组指标条件
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalGroupIndicatorPreconditionDTO {

    private static final long serialVersionUID = 1L;

    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    @NotNull(message = "数据类型不能为空")
    private DataTypeEnum dataType;

    /**
     * 指标类型（1.指标，2，考评指标）
     */
    @NotNull(message = "指标类型不能为空")
    private IndicatorCodeTypeEnum codeType;

    /**
     * 指标编码
     */
    @NotNull(message = "指标编码不能为空")
    private String code;

    /**
     * 指标库名称
     */
    @NotNull(message = "指标库名称不能为空")
    private String name;

    /**
     * 目标计算类型;1: 目标值计算 2:最低要求计算
     */
    private TargetCalcTypeEnum targetCalcType;

    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额 4:台数
     */
    @NotNull(message = "目标值类型不能为空")
    private TargetTypeEnum targetType;

    /**
     * 目标值
     */
    private BigDecimal targetValue;

    /**
     * 排序
     */
    @NotNull(message = "排序不能为空")
    private Integer sort;

    /**
     * 条件阶梯
     */
    @Valid
    private List<EvalGroupIndicatorPreconditionLaddersDTO> ladders;

}
