package cn.fw.morax.domain.dto.eval;

import cn.fw.morax.domain.enums.*;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;

/**
 * <p>
 * 考评奖惩前置条件
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalGroupRewardPreconditionDTO {

    private static final long serialVersionUID = 1L;

    /**
     * 条件值类型; 1:满足目标 2:满足排名率 3:金额
     */
    @NotNull(message = "条件值类型不能为空")
    private ConditionTypeEnum condType;

    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    private DataTypeEnum dataType;

    /**
     * 编码类型（1. 指标，2.考评指标）
     */
    @NotNull(message = "编码类型不能为空")
    private IndicatorCodeTypeEnum codeType;

    /**
     * 项目编码
     */
    @NotNull(message = "项目编码不能为空")
    private String code;

    /**
     * 项目名称
     */
    @NotNull(message = "项目名称不能为空")
    private String name;

    /**
     * 目标值类型; 1:无 2:百分比 3:金额 4:台数
     */
    @NotNull(message = "目标值类型不能为空")
    private TargetTypeEnum targetType;

    /**
     * 目标计算类型;1: 目标值计算 2:最低要求计算
     */
    private TargetCalcTypeEnum targetCalcType;

    /**
     * 目标值
     */
    private BigDecimal targetValue;

    /**
     * 条件值
     */
    @NotNull(message = "条件值不能为空")
    private BigDecimal condValue;

    /**
     * 排序
     */
    @NotNull(message = "排序不能为空")
    private Integer sort;

}
