package cn.fw.morax.domain.dto.kpi;

import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.Indicators;
import cn.fw.morax.domain.enums.*;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @date : 2022-4-2
 * @desc : 指标库
 */
@Data
public class IndicatorsDTO {

    private Long id;

    /**
     * 指标名称
     */
    @NotBlank(message = "指标名称不能为空")
    @Length(max = 255, message = "指标名称超出最大长度限制")
    private String indicatorName;

    /**
     * 归属系统id
     */
    @NotNull(message = "归属系统不能为空")
    private Long sysId;

    /**
     * 归属系统名称
     */
    @NotNull(message = "归属系统名称不能为空")
    private String sysName;

    /**
     * 适用角色类型;1: 全部角色 2:全部管理角色 3:自定义
     */
    @NotNull(message = "适用角色类型不能为空")
    private ApplicableRoleEnum roleType;

    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    @NotNull(message = "数据类型不能为空")
    private DataTypeEnum dataType;

    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额
     */
    @NotNull(message = "绩效目标值类型不能为空")
    private TargetTypeEnum targetType;

    /**
     * 适用范围类型; 1. 绩效组, 2.薪酬组, 3. 考评组
     */
    @NotNull(message = "适用范围类型不能为空")
    @Size(min = 1, message = "适用范围不能为空")
    private List<ApplicableTypeEnum> applyTypes;

    /**
     * 数据范围范围类型; 1. 人员, 2.门店
     */
    @NotNull(message = "数据范围范围类型不能为空")
    @Size(min = 1, message = "数据范围范围类型不能为空")
    private List<DataScopeTypeEnum> dataScopeTypes;

    /**
     * 内部指标
     * 0:否 1:是
     */
//    @NotNull(message = "内部指标不能为空")
//    private Boolean innerIndicator;

    /**
     * 适用角色编码
     */
//    @NotNull(message = "适用角色编码不能为空")
//    @Size(min = 1, message = "适用角色编码不能为空")
    private List<String> roleCodes;

    /**
     * 适用角色名称
     */
//    @NotNull(message = "适用角色名称不能为空")
//    @Size(min = 1, message = "适用角色名称不能为空")
    private List<String> roleNames;

    /**
     * 报表路径
     */
    @Length(max = 255, message = "报表路径超出最大长度限制")
    private String reportPath;

//    /**
//     * 启用/禁用;
//     * 0:禁用 1:启用
//     */
//    @NotNull(message = "启用禁用不能为空")
//    private Boolean enable;

    /**
     * 转换为db对象
     * @return
     */
    public Indicators convertToPo(){
        Indicators indicators = Indicators.builder()
                .indicatorName(this.getIndicatorName())
                .enable(Boolean.FALSE)
                .roleType(this.getRoleType())
                .dataType(this.getDataType())
                .targetType(this.targetType)
                .sysId(this.getSysId())
//                .enable(this.getEnable())
//                .innerIndicator(this.innerIndicator)
                .sysName(this.getSysName())
                .build();
        if (ApplicableRoleEnum.PORTION.equals(this.getRoleType())) {
            if (PublicUtil.isNotEmpty(this.getRoleCodes())) {
                indicators.setRoleCodes(this.getRoleCodes());
            }
            if (PublicUtil.isNotEmpty(this.getRoleNames())) {
                indicators.setRoleNames(this.getRoleNames());
            }
        }
        if (PublicUtil.isEmpty(this.getId())) {
            indicators.setId(this.getId());
        }
        List<Integer> applyTypes = this.applyTypes.stream().map(applyType -> applyType.getValue()).collect(Collectors.toList());
        indicators.setApplyTypes(applyTypes);
        List<Integer> dataScopeTypes = this.dataScopeTypes.stream().map(applyType -> applyType.getValue()).collect(Collectors.toList());
        indicators.setDataScopeTypes(dataScopeTypes);
        return indicators;
    }

}