package cn.fw.morax.domain.dto.kpi;

import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.morax.domain.enums.EffectMonthEnum;
import cn.fw.morax.domain.enums.SettingDraftStatusEnum;
import cn.fw.morax.domain.enums.StarEvaluationEnum;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 绩效组配置
 */
@Data
public class KpiGroupDTO {

    /**
     * 绩效组id（前一次）
     */
    private Long id;
    /**
     * 草稿id
     */
    private Long draftId;
    /**
     * 名称
     */
    @NotBlank(message = "名称不能为空")
    @Length(max = 255, message = "名称超出最大限制")
    private String name;

    /**
     * 绩效组编码
     */
    private String kgc;

    /**
     * 岗位id
     */
    @NotNull(message = "岗位id不能为空")
    private Long postId;

    /**
     * 岗位名称
     */
    @NotBlank(message = "岗位名称不能为空")
    private String postName;

    /**
     * 适用门店ids
     */
    @NotNull(message = "适用门店id不能为空")
    @Size(min = 1, message = "适用门店id不能为空")
    private List<Long> shopIds;

    /**
     * 适用门店名称
     */
    @NotNull(message = "适用门店名称不能为空")
    @Size(min = 1, message = "适用门店名称不能为空")
    private List<String> shopNames;

    /**
     * 生效时间
     */
    private LocalDate beginTime;

    @NotNull(message = "生效时间类型不能为空")
    private EffectMonthEnum beginTimeType;

    /**
     * 调整原因
     */
    @NotBlank(message = "调整原因不能为空")
    @Length(max = 512, message = "调整原因超出最大限制")
    private String reason;

    /**
     * 附件;fid逗号隔开
     */
    private List<String> attachment;

    /**
     * 星级评定方式;1:绩效得分率 2:人员百分比
     */
    @NotNull(message = "星级评定方式不能为空")
    private StarEvaluationEnum starEvaluationType;

    /**
     * 集团id
     force     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_NAME)
    private String userName;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_ID)
    private Long userId;

    /**
     * 强制保存
     */
    @NotNull(message = "是否强制保存不能为空")
    private Boolean force;

    /**
     * 绩效组指标项
     */
    @NotNull(message = "绩效组指标项不能为空")
    @Size(min = 1, message = "绩效组指标项不能为空")
    @Valid
    private List<KpiGroupIndicatorDTO> indicators;

    /**
     * 星级评定阶梯
     */
    @Valid
    @Size(min = 1, message = "星级评定阶梯不能为空")
    @NotNull(message = "星级评定阶梯不能为空")
    private List<KpiStarLaddersDTO> starLadders;

    /**
     * 撤销D级得分率
     */
    private BigDecimal revokedScoreRatio;

    /**
     * 审批单号
     */
    private String approvalNo;

    /**
     * 包含指标数量
     */
    private Integer indicatorNum;

    /**
     * 草稿状态 1:未发布 2:发布待审批 3:审批拒绝 4:审批同意 5:撤销审批
     */
    private SettingDraftStatusEnum draftStatus;

}