package cn.fw.morax.domain.dto.kpi;

import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.morax.domain.enums.StarEvaluationEnum;
import lombok.*;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 绩效组排名配置
 * </p>
 *
 * @author jiangchao
 * @since 2022-09-26
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class KpiGroupRankDto {

    /**
     * id
     */
    private Long id;

    /**
     * 组合名称
     */
    @NotBlank(message = "组合名称不能为空")
    @Length(max = 255, message = "组合名称超出最大限制")
    private String name;

    /**
     * 星级评定方式;1:绩效得分率 2:人员百分比
     */
    @NotNull(message = "星级评定方式不能为空")
    private StarEvaluationEnum starEvaluationType;

    /**
     * 集团id
     force     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_NAME)
    private String userName;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_ID)
    private Long userId;

    /**
     * 撤销D级得分率
     */
    private BigDecimal revokedScoreRatio;

    /**
     * 绩效组编码
     */
    @NotNull(message = "绩效组编码不能为空")
    @Size(min = 1, message = "绩效组编码不能为空")
    private List<String> kgcs;

    /**
     * 星级评定阶梯
     */
    @Valid
    @Size(min = 1, message = "星级评定阶梯不能为空")
    @NotNull(message = "星级评定阶梯不能为空")
    private List<KpiGroupRankStarLaddersDTO> starLadders;

}
