package cn.fw.morax.domain.dto.kpi;

import cn.fw.common.web.annotation.LoginContextField;
import lombok.Data;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 绩效星级规则
 */
@Data
public class KpiStarRuleDTO {

    /**
     * 绩效星级规则id
     */
//    private Long id;

    /**
     * 滚动月数
     */
    @NotNull(message = "滚动月数不能为空")
    @Max(value = 60, message = "滚动月数不能超60")
    @Min(value = 0, message = "滚动月数不能小于0")
    private Integer rollMonth;

    /**
     * 强制离职考核周期
     */
    @NotNull(message = "强制离职考核月数不能为空")
    @Max(value = 60, message = "强制离职考核月数不能超60")
    @Min(value = 0, message = "强制离职考核月数不能小于0")
    private Integer forcedTurnoverCycle;

    /**
     * 强制离职周期内最大累计D级数
     */
    @NotNull(message = "最大累计D级次数不能为空")
    @Max(value = 60, message = "最大累计D级次数不能超过60")
    @Min(value = 0, message = "最大累计D级次数不能小于0")
    private Integer maxUnqualifiedTimes;

    /**
     * 连续月份;满足次条件减少一次D级
     */
    @NotNull(message = "减少D级条件连续月份不能为空")
    @Max(value = 60, message = "减少D级条件连续月份不能超过60")
    @Min(value = 0, message = "减少D级条件连续月份不能小于0")
    private Integer continuousMonthly;

    /**
     * 最小得分率
     */
    @NotNull(message = "最小得分率不能为空")
    @Max(value = 100, message = "最小得分率不能超100")
    @Min(value = 0, message = "最小得分率不能小于0")
    private BigDecimal minScoreRatio;

    /**
     * 集团id
     *
     * @ignore
     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;
}