package cn.fw.morax.domain.dto.kpi;

import cn.fw.common.validator.EnumValue;
import cn.fw.morax.domain.enums.IndicatorValueTypeEnum;
import lombok.Data;
import org.hibernate.validator.constraints.ScriptAssert;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Past;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 人员指标值
 *
 * @author : kurisu
 * @version : 2.0
 * @className : UserIndicatorDTO
 * @description : 人员指标值
 * @date : 2022-12-22 14:42
 */
@Data
@ScriptAssert.List({
        @ScriptAssert(lang = "javascript", script = "_this.checkValueType()", message = "只支持简单类型的指标"),
})
public class UserIndicatorDTO {
    /**
     * 指标编码
     */
    @NotEmpty(message = "指标编码不能为空")
    private String indicatorCode;
    /**
     * 人员id
     */
    @NotNull(message = "人员id不能为空")
    private Long userId;
    /**
     * 指标值类型
     */
    @NotNull(message = "指标值类型不能为空")
    @EnumValue(enumClass = IndicatorValueTypeEnum.class, message = "指标值枚举值不正确")
    private Integer valueType;
    /**
     * 指标对应值
     */
    @NotNull(message = "指标值不能为空")
    private BigDecimal value;
    /**
     * 数据日期
     */
    @NotNull(message = "数据日期不能为空")
    @Past(message = "数据日期必须为过去的时间")
    private Date dataDate;
    /**
     * 集团id
     */
    @NotNull(message = "集团id不能为空")
    private Long groupId;

    public boolean checkValueType() {
        return IndicatorValueTypeEnum.CONDITION.getValue().equals(valueType) || IndicatorValueTypeEnum.WORK_AGE.getValue().equals(valueType);
    }
}
