package cn.fw.morax.domain.dto.query;

import cn.fw.common.page.BasePageQuery;
import cn.fw.common.web.annotation.LoginContextField;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;

import javax.validation.constraints.NotNull;
import java.time.YearMonth;
import java.util.List;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : KpiPoolQueryDTO
 * @description : 考评池查询条件
 * @date : 2022-05-05 11:50
 */
@Data
@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = true)
public class EvalPoolQueryDTO extends BasePageQuery {
    /**
     * 管理员工id
     */
    private Long managerUserId;
    /**
     * 员工id
     */
    private Long userId;
    /**
     * 员工名称
     */
    private String userName;
    /**
     * 月度
     *
     * @ignore
     */
    private YearMonth monthly;
    /**
     * 岗位id
     */
    private Long postId;
    /**
     * 岗位名称
     */
    private String postName;
    /**
     * 门店id
     */
    private Long shopId;
    /**
     * 门店名称
     */
    private String shopName;
    /**
     * 1升序 2降序
     */
    private Integer order;
    /**
     * 排序字段名称 eg. shopId、startLevel...
     */
    private String orderAtt;

    /**
     * 当前登录人id
     *
     * @ignore
     */
    @LoginContextField(value = LoginContextField.Name.USER_ID)
    private Long currentUserId;

    /**
     * 当前登录人集团id
     *
     * @ignore
     */
    @LoginContextField(value = LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 管理员工列表
     *
     * @ignore
     */
    private List<Long> managerStaffIds;

    /**
     * 管理门店列表
     *
     * @ignore
     */
    private List<Long> managerShopIds;

    /**
     * 排序字符串
     *
     * @ignore
     */
    private String orderString;

    /**
     * 查询员工列表
     *
     * @ignore
     */
    private String queryStaffs;

//    /**
//     * 管理员工列表
//     *
//     * @ignore
//     */
//    private List<Long> queryStaffList;
}
