package cn.fw.morax.domain.dto.salary;

import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import cn.fw.morax.domain.vo.kpi.KpiGroupIndicatorTargetVO;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 薪酬组薪酬项目
 */
@Data
public class SalaryGroupProjectDTO {
    /**
     * 名称
     */
    @NotBlank(message = "名称不能为空")
    private String name;

    /**
     * 计算方式;1: 星级 2:固定金额 3:阶梯提成 4:车系提成 99:动态
     */
    @NotNull(message = "计算方式不能为空")
    private SalaryCalMethodEnum calMethod;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    @NotNull(message = "数据类型不能为空")
    private DataTypeEnum laddersType;

    /**
     * 台阶参数别名
     */
    public String ladderParamAlias;

    /**
     * 提成参数别名
     */
    public String commissionParamAlias;

    /**
     * 前置条件别名
     */
    public String preconditionAlias;

    /**
     * 薪酬项配置
     */
    @NotEmpty(message = "薪酬项配置不能为空")
    @Valid
    private List<SalaryGroupProjectSettinDTO> settings;

    /**
     * 台阶参数
     */
    @Valid
    private List<SalaryGroupProjectParamDTO> ladderParams;

    /**
     * 提成参数
     */
//    @NotNull(message = "提成参数不能为空")
//    @Size(min = 1, message = "提成参数不能为空")
    @Valid
    private List<SalaryGroupProjectParamDTO> commissionParams;

    /**
     * 条件
     */
    @Valid
    private List<SalaryGroupProjectPreconditionDTO> conds;

    /**
     * 指标目标
     */
    private List<KpiGroupIndicatorTargetVO> targets;
}