package cn.fw.morax.domain.dto.salary;

import cn.fw.morax.domain.enums.ApplicableRoleEnum;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-2
 * @desc : 薪酬项
 */
@Data
public class SalaryProjectDTO {

    private Long id;

    /**
     * 薪酬项名称
     */
    @NotBlank(message = "指标名称不能为空")
    @Length(max = 255, message = "指标名称超出最大长度限制")
    private String salaryName;

    /**
     * 可选计算方式;1: 星级 2:固定金额 3:阶梯提成(单) 4:按量 5: 固定提成 6:阶梯提成(总)
     * 逗号隔开
     */
    @NotNull(message = "计算方式不能为空")
    @Size(min = 1, message = "计算方式不能为空")
    private List<Integer> optionalMethod;

    /**
     * 可见的;对用户可见 只有对用户可见的才能配置薪酬项，否则为系统使用的项目
     */
    @NotNull(message = "计算方式不能为空")
    private Boolean visible;

    /**
     * 适用角色类型;1: 全部角色 2:全部管理角色 3:自定义
     */
    @NotNull(message = "适用角色类型不能为空")
    private ApplicableRoleEnum roleType;

    /**
     * 适用角色编码
     */
    private List<String> roleCodes;

    /**
     * 适用角色名称
     */
    private List<String> roleNames;
}