package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 适用范围类型
 *
 * @author kurisu
 */
public enum ApplicableTypeEnum implements IEnum<Integer> {
    /**
     * 适用范围类型; 1. 绩效组, 2.薪酬组, 3. 考评组
     */
    KPI(1, "绩效组"),
    SALARY(2, "薪酬组"),
    EVAL(3, "考评组"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ApplicableTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ApplicableTypeEnum ofValue(final Integer value) {
        for (final ApplicableTypeEnum _enum : ApplicableTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ApplicableTypeEnum _enum : ApplicableTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }
}
