package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : ApprovalTypeEnum
 * @description : 审批类型枚举
 * @date: 2022-04-06 14:18
 */
public enum ApprovalTypeEnum implements IEnum<Integer> {
    /**
     * 审批业务类型;1:绩效配置 2:薪酬配置 3:绩效星级申诉 4：岗位最低收入补偿 5：考评
     */
    KPI(1, "绩效配置"),
    SALARY(2, "薪酬配置"),
    COMPLAIN(3, "绩效星级申诉"),
    INCOME_COMPENSATION(4, "岗位最低收入补偿"),
    EVAL(5, "考评"),
    EVAL_REWARD_DIST(6, "考评奖惩分配"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ApprovalTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ApprovalTypeEnum ofValue(final Integer value) {
        for (final ApprovalTypeEnum _enum : ApprovalTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ApprovalTypeEnum _enum : ApprovalTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
