package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : SettingStatusEnum
 * @description : 配置状态枚举
 * @date: 2022-04-06 10:41
 */
public enum DataImportStatusEnum implements IEnum<Integer> {
    /**
     * 状态；1.待推送待办 2.导入中 2.角色无授权人 3. 角色无授权人 4. 完成
     */
    WAIT_PUSH(1, "待推送待办"),
    IMPORTING(2, "导入中"),
    NO_MATCH(3, "角色无授权人"),
    COMPLETION(4, "完成"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    DataImportStatusEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static DataImportStatusEnum ofValue(final Integer value) {
        for (final DataImportStatusEnum _enum : DataImportStatusEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final DataImportStatusEnum _enum : DataImportStatusEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
