package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : IndicatorValueTypeEnum
 * @description : 维度类型枚举
 * @date : 2022-04-25 11:23
 */
public enum DimensionTypeEnum implements IEnum<Integer> {
    /**
     * 维度类型 1：人员 2：门店
     */
    STAFF(1, "人员"),
    SHOP(2, "门店"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    DimensionTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static DimensionTypeEnum ofValue(final Integer value) {
        for (final DimensionTypeEnum salaryValueEnum : DimensionTypeEnum.values()) {
            if (salaryValueEnum.value.equals(value)) {
                return salaryValueEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final DimensionTypeEnum salaryValueEnum : DimensionTypeEnum.values()) {
            if (salaryValueEnum.value.equals(value)) {
                return salaryValueEnum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @Override
    @JsonValue
    public Integer getValue() {
        return value;
    }
}
