package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : IndicatorValueTypeEnum
 * @description : 指标数据类型枚举
 * @date : 2022-04-25 11:23
 */
public enum IndicatorValueTypeEnum implements IEnum<Integer> {
    /**
     * 指标数据类型
     */
    RATIO(1, "百分比"),
    QUANTITY(2, "数量/金额"),
    CONDITION(3, "条件值"),
    WORK_AGE(5, "工龄"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    IndicatorValueTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static IndicatorValueTypeEnum ofValue(final Integer value) {
        for (final IndicatorValueTypeEnum salaryValueEnum : IndicatorValueTypeEnum.values()) {
            if (salaryValueEnum.value.equals(value)) {
                return salaryValueEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final IndicatorValueTypeEnum salaryValueEnum : IndicatorValueTypeEnum.values()) {
            if (salaryValueEnum.value.equals(value)) {
                return salaryValueEnum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @Override
    @JsonValue
    public Integer getValue() {
        return value;
    }
}
