package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : KpiIgnoreCauseEnum
 * @description : 不计入绩效考核原因
 * @date: 2022-04-06 11:16
 */
public enum KpiIgnoreCauseEnum implements IEnum<Integer> {
    /**
     * 待定
     */
    PROBATION(1, "试用期不计算绩效"),
    TURN_POSITIVE_AFTER(2, "转正两个月内不计算绩效"),
    TRANSFER_NEW_POSITION_AFTER(3, "获得正式岗位后不足2个月人员"),
    APPLY_NO_ASSESSMENT(4, "主动申请不计算绩效"),
    AWAIT_JOB_DURATION(5, "待岗期间"),
    AWAIT_JOB_AFTER(6, "待岗后两个月内不计算绩效"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    KpiIgnoreCauseEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static KpiIgnoreCauseEnum ofValue(final Integer value) {
        for (final KpiIgnoreCauseEnum _enum : KpiIgnoreCauseEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final KpiIgnoreCauseEnum _enum : KpiIgnoreCauseEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
