package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryTypeEnum
 * @description : 薪资类型
 * @date : 2022-04-24 17:09
 */
public enum SalaryTypeEnum implements IEnum<Integer> {
    /**
     * 薪资类型
     * 1:绩效提成 2:内部奖励 3:内部罚款 4:福利补贴 5:社保扣款 6:试用期工资 7:个税
     */
    ROYALTIES(1, "绩效提成", true),
    AWARD(2, "内部奖励", true),
    PENALTY(3, "内部罚款", false),
    SUBSIDY(4, "福利补贴", true),
    NECESSARY(5, "社保扣款", false),
    PROBATION(6, "试用期工资", true),
    PERSON_TAX(7, "个税", false),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;
    /**
     * 加算
     */
    @Getter
    private final boolean plus;

    SalaryTypeEnum(final Integer value, final String name, final boolean plus) {
        this.value = value;
        this.name = name;
        this.plus = plus;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static SalaryTypeEnum ofValue(final Integer value) {
        for (final SalaryTypeEnum _enum : SalaryTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final SalaryTypeEnum _enum : SalaryTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
