package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : ScoreWayEnum
 * @description : 绩效得分方式
 * @date: 2022-04-06 11:04
 */
public enum ScoreWayEnum implements IEnum<Integer> {
    /**
     * 绩效得分方式;1: 阶梯得分 2:普通得分
     */
    LADDER(1, "阶梯得分"),
    NORMAL(2, "普通得分"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ScoreWayEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ScoreWayEnum ofValue(final Integer value) {
        for (final ScoreWayEnum _enum : ScoreWayEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ScoreWayEnum _enum : ScoreWayEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
