package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 目标计算类型
 *
 * @author kurisu
 */
public enum TargetCalcTypeEnum implements IEnum<Integer> {

    /**
     * 目标值计算
     */
    TARGET_VALUE(1, "目标值计算"),
    /**
     * 最低要求计算
     */
    MINIMUM(2, "最低要求计算"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    TargetCalcTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static TargetCalcTypeEnum ofValue(final Integer value) {
        for (final TargetCalcTypeEnum _enum : TargetCalcTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final TargetCalcTypeEnum _enum : TargetCalcTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }
}
