package cn.fw.morax.domain.vo.eval;

import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.EvalScopeEnum;
import cn.fw.morax.domain.enums.RankTypeEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;

/**
 * <p>
 * 考评明细
 * </p>
 *
 * @author jiangchao
 * @since 2023-02-11
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalPoolRewardDetailVO {

    private static final long serialVersionUID = 1L;

    /**
     *  id
     */
    private Long id;
    /**
     * 考评池id
     */
    private Long evalPoolId;

    /**
     * 考评范围; 1:门店考评 2:人员考评
     */
    private EvalScopeEnum scopeType;

    /**
     * 考评项配置id
     */
    private Long evalGroupRewardId;

    /**
     * 预计收入
     */
    private BigDecimal amount;

    /**
     * 日期
     */
//    private LocalDate dataDate;

    /**
     * 集团id
     */
    private Long groupId;

    /**
     * 考评奖惩指标名称
     */
    @NotBlank(message = "考评奖惩指标名称不能为空")
    @Length(max = 255, message = "考评奖惩指标名称超出最大限制")
    private String name;

    /**
     * 计算方式；1: 排名正负激励 2:固定值 3:阶梯 4:总金额占比 5.阶梯翻倍
     */
    @NotNull(message = "计算方式不能为空")
    private EvalRewardCalMethodEnum calMethod;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    private DataTypeEnum laddersType;

    /**
     * 排名方式;1:排名序号 2:排名人数百分比
     */
    private RankTypeEnum rankType;

    /**
     * 金额
     */
//    @NotNull(message = "金额不能为空")
    private BigDecimal money;

    /**
     * 追加奖励 XX 元/台，超额奖励用
     */
    private BigDecimal extraMoney;

    /**
     * 命中的台阶指标值
     */
    private BigDecimal hitLadderValue;
    /**
     * 命中的提成指标值
     */
    private BigDecimal hitCommissionValue;
    /**
     * 占比
     */
    private BigDecimal proportionValue;

    /**
     * 台阶参数别名
     */
    public String ladderParamAlias;

    /**
     * 提成参数别名
     */
    public String commissionParamAlias;

    /**
     * 前置条件别名
     */
    public String preconditionAlias;

    /**
     * 提成参数
     */
    @Valid
    private List<EvalGroupRewardParamVO> commissionParams;

    /**
     * 台阶参数
     */
    @Valid
    private List<EvalGroupRewardParamVO> ladderParams;

    /**
     * 阶梯
     */
    @Valid
    private List<EvalGroupRewardLaddersVO> ladders;

    /**
     * 条件
     */
    @Valid
    private List<EvalGroupRewardPreconditionVO> conds;

}
