package cn.fw.morax.domain.vo.salary;

import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.ParamTypeEnum;
import cn.fw.morax.domain.enums.TargetCalcTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * <p>
 * 薪酬组薪酬项目参数
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-05
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class SalaryGroupProjectParamVO {

    private static final long serialVersionUID = 1L;

    /**
     * 薪酬组id
     */
    private Long id;

    /**
     * 薪酬组id
     */
    private Long salaryGroupId;

    /**
     * 薪酬组项目id
     */
    private Long salaryGroupProjectId;

    /**
     * 参数类型（1.台阶，2，提成）
     */
    private ParamTypeEnum paramType;

    /**
     * 指标编码
     */
    private String indicatorCode;

    /**
     * 指标库名称
     */
    private String indicatorName;


    /**
     * 目标值类型; 1:无 2:百分比 3:金额 4:台数
     */
    private TargetTypeEnum targetType;

    /**
     * 目标计算类型;1: 目标值计算 2:最低要求计算
     */
    private TargetCalcTypeEnum targetCalcType;

    /**
     * 目标值
     */
    private BigDecimal targetValue;

    /**
     * 指标值
     */
    private BigDecimal indicatorValue;

    /**
     * 达成值
     */
    private BigDecimal reachValue;

    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    private DataTypeEnum dataType;

    /**
     * 占比
     */
    private BigDecimal proportion;

    /**
     * 是否封顶
     */
    private Boolean cap;

    /**
     * 排序
     */
    private Integer sort;

    /**
     * 是否基础工资
     */
    public Boolean baseWage;

    /**
     * 处理百分数
     */
    public void processPercent() {
        this.setProportion(this.getProportion().multiply(Constant.ONE_HUNDRED));
        if (TargetTypeEnum.RATIO.equals(this.getTargetType())) {
            this.setTargetValue(this.getTargetValue().multiply(Constant.ONE_HUNDRED));
        }
    }


}
