package cn.fw.morax.domain.vo.salary;

import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import lombok.Data;

import java.math.BigDecimal;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 薪酬项目配置
 */
@Data
public class SalaryGroupProjectSettinVO implements Comparable<SalaryGroupProjectSettinVO>{
    /**
     * 主键id
     */
    private Long id;
    /**
     * 薪酬项目配置id
     */
    private Long salaryGroupProjectId;
    /**
     * 用户薪酬项命中记录id
     */
    private Long salaryProjectHitLogId;
    /**
     * 阶梯key;新车销售按车系计算是 对应车系id其他则为绩效项id
     */
    private Long stairKey;
    /**
     * 阶梯key对应描述
     */
    private String stairKeyDesc;
    /**
     * 计算相关值
     */
    private BigDecimal stairValue;
    /**
     * 阶梯值下限
     */
    private BigDecimal stairMin;
    /**
     * 阶梯值上限;阶梯值上限
     */
    private BigDecimal stairMax;

    /**
     * 是否命中
     */
    private Boolean hit;
    /**
     * 薪酬值
     */
    private BigDecimal salaryValue;
    /**
     * 命中的台阶指标值
     */
    private BigDecimal hitLadderValue;
    /**
     * 命中的提成指标值(车系：车系销售台数)
     */
    private BigDecimal hitCommissionValue;

    @Override
    public int compareTo(SalaryGroupProjectSettinVO other) {
        if (PublicUtil.isNotEmpty(this.stairMax)) {
            return this.stairMax.compareTo(other.getStairMax());
        } else {
            return this.id.compareTo(other.getId());
        }
    }

    /**
     * 处理百分数
     */
    public void processPercent() {
        if (PublicUtil.isNotEmpty(this.getStairMin())) {
            this.setStairMin(this.getStairMin().multiply(Constant.ONE_HUNDRED));
        }
        if (PublicUtil.isNotEmpty(this.getStairMax())) {
            this.setStairMax(this.getStairMax().multiply(Constant.ONE_HUNDRED));
        }
//        if (PublicUtil.isNotEmpty(this.getHitCommissionValue())) {
//            this.setHitCommissionValue(this.getHitCommissionValue().multiply(Constant.ONE_HUNDRED));
//        }
//        if (PublicUtil.isNotEmpty(this.getHitLadderValue())) {
//            this.setHitLadderValue(this.getHitLadderValue().multiply(Constant.ONE_HUNDRED));
//        }
    }

}