package cn.fw.morax.server.controller.api;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.sdk.api.IndicatorReportService;
import cn.fw.morax.sdk.dto.CustomList;
import cn.fw.morax.sdk.dto.common.IndicatorShopValueReq;
import cn.fw.morax.sdk.dto.common.IndicatorValueReq;
import cn.fw.morax.service.biz.kpi.IndicatorBizService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.morax.common.utils.MessageFormatUtil.MessageFormatTransfer;

/**
 * @author : kurisu
 * @version : 2.0
 * @className : IndicatorReportServiceImpl
 * @description : 指标上报类
 * @date : 2022-12-22 10:05
 */
@Slf4j
@RestController
@Validated
@RequiredArgsConstructor
@RequestMapping("/api/morax/indicator")
public class IndicatorReportServiceImpl implements IndicatorReportService {
    private final static int MAX_SIZE = 1000;
    private final IndicatorBizService indicatorBizService;

    @PostMapping("/report/up")
    @ControllerMethod("上报人员指标数据数据")
    @Override
    public Message<Void> indicatorReport(@Valid @RequestBody CustomList<IndicatorValueReq> userIndicatorList) {
        BV.isTrue(userIndicatorList.size() <= MAX_SIZE, () -> MessageFormatTransfer("最多支持一次性处理{0}条任务", MAX_SIZE));
        indicatorBizService.reportData(userIndicatorList);
        return Message.success();
    }

    @PostMapping("/report-shop/up")
    @ControllerMethod("上报门店指标数据数据")
    @Override
    public Message<Void> indicatorShopReport(@Valid @RequestBody CustomList<IndicatorShopValueReq> userIndicatorList) {
        BV.isTrue(userIndicatorList.size() <= MAX_SIZE, () -> MessageFormatTransfer("最多支持一次性处理{0}条任务", MAX_SIZE));
        indicatorBizService.reportShopData(userIndicatorList);
        return Message.success();
    }
}
