package cn.fw.morax.server.controller.api;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.sdk.api.IKpiReportService;
import cn.fw.morax.sdk.dto.kpi.KpiGroupUserQuery;
import cn.fw.morax.sdk.dto.kpi.KpiGroupUserResult;
import cn.fw.morax.sdk.dto.kpi.KpiPointRewardReq;
import cn.fw.morax.service.biz.kpi.KpiReportService;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 绩效上报api
 *
 * @author : kurisu
 * @version : 2.0
 * @className : KpiReportServiceImpl
 * @description : 绩效上报api
 * @date : 2022-04-11 17:21
 */

@Slf4j
@RestController
@Validated
@RequiredArgsConstructor
@RequestMapping("/api/morax/kpi")
public class KpiReportServiceImpl implements IKpiReportService {

    @Getter
    private final int maxSize = 1000;
    private final KpiReportService kpiReportService;

    @Deprecated
    @GetMapping("/query/user")
    @Override
    @ControllerMethod("查询绩效组人员")
    public Message<List<KpiGroupUserResult>> queryKpiGroupUser(@Validated({KpiGroupUserQuery.Query.class}) @SpringQueryMap KpiGroupUserQuery kpiGroupUserQuery) {
        return success(kpiReportService.getIndicatorUser(kpiGroupUserQuery));
    }

    @Deprecated
    @GetMapping("/real-time/query/user")
    @Override
    @ControllerMethod("实时查询绩效组人员")
    public Message<List<KpiGroupUserResult>> realTimeQueryKpiGroupUser(@Validated({KpiGroupUserQuery.RealTimeQuery.class}) @SpringQueryMap KpiGroupUserQuery kpiGroupUserQuery) {
        return success(kpiReportService.getRealTimeIndicatorUser(kpiGroupUserQuery));
    }

    /**
     * 绩效分奖惩上报
     *
     * @param kpiPointRewardReq 绩效分奖惩上报数据
     * @return void
     */
    @PostMapping("/point/reward")
    @Override
    @ControllerMethod("绩效分奖惩上报")
    public Message<Void> kpiPointReward(@Valid @RequestBody KpiPointRewardReq kpiPointRewardReq) {
        kpiReportService.reportKpiReward(kpiPointRewardReq);
        return success();
    }
}
