package cn.fw.morax.server.controller.app;


import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.eval.EvalDTO;
import cn.fw.morax.domain.dto.eval.EvalGroupIndicatorRankDTO;
import cn.fw.morax.domain.dto.eval.EvalRewardDistDTO;
import cn.fw.morax.domain.dto.query.EvalPoolQueryDTO;
import cn.fw.morax.domain.vo.SelectorVO;
import cn.fw.morax.domain.vo.eval.*;
import cn.fw.morax.domain.vo.kpi.KpiIndicatorRankVO;
import cn.fw.morax.service.biz.SettingDraftBizService;
import cn.fw.morax.service.biz.eval.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.annotation.IgnoreUserToken;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.Objects;
import java.util.Set;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 考评
 *
 * @author jiangchao
 * @since 2022-11-09
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/app/eval")
public class EvalController {

    private final EvalPoolService evalPoolService;
    private final EvalGroupPoolService evalGroupPoolService;
    private final EvalGroupBizService evalGroupBizService;
    private final EvalGroupDataService evalGroupDataService;
    private final SettingDraftBizService settingDraftBizService;
    private final EvalRewardService evalRewardService;
    private final EvalIndicatorValueBizService evalIndicatorValueBizService;

    /**
     * 考评详情
     *
     * @param id
     * @return
     */
    @GetMapping("/group")
    @ControllerMethod("考评详情")
    public Message<EvalVO> evalGroupDetail(@NotNull(message = "考评组id不能为空") Long id) {

        return success(evalGroupBizService.evalDetail(id));
    }

    /**
     * 考评草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/eval-detail")
    @ControllerMethod("考评草稿详情")
    public Message<EvalDTO> draftEvalDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftEvalDetail(id));
    }

    /**
     * 考评数据录入详情
     *
     * @param id
     * @return
     */
    @GetMapping("/report-detail")
    @ControllerMethod("考评数据录入详情")
    public Message<EvalReportVO> reportDetail(@NotNull(message = "录入详情id不能为空") Long id) {

        return success(evalIndicatorValueBizService.reportDetail(id));
    }

    /**
     * 奖惩分配详情
     *
     */
    @GetMapping("/reward-dist")
    @ControllerMethod("奖惩分配详情")
    public Message<EvalRewardDistVO> rewardDistDetail(@NotNull(message = "奖惩分配id不能为空") Long id) {
        return success(evalRewardService.rewardDistDetail(id));
    }

    /**
     * 奖惩分配
     *
     */
    @PostMapping("/reward-dist")
    @ControllerMethod("奖惩分配")
    public Message<Void> rewardDist(@Validated @RequestBody EvalRewardDistDTO rewardDist,
                                    @CurrentUser LoginAuthBean currentUser) {
        evalRewardService.rewardDist(rewardDist, currentUser);
        return success();
    }



}
