package cn.fw.morax.server.controller.app;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.kpi.KpiStarSpecialRuleDTO;
import cn.fw.morax.domain.vo.kpi.KpiStarSpecialRuleVO;
import cn.fw.morax.domain.vo.salary.StaffSelectVO;
import cn.fw.morax.service.biz.kpi.KpiStarSpecialRuleBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import java.time.YearMonth;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 绩效星级调整
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/kpi")
@IgnoreAuth
public class KpiStaffController {

    private final KpiStarSpecialRuleBizService specialRuleBizService;

    /**
     * 申请绩效星级特殊调整
     *
     * @param dto dto
     * @return
     */
    @PostMapping("/star-adjustment")
    @ControllerMethod("申请绩效星级特殊调整")
    public Message<Void> applyKpiStarAdjustment(@Validated @RequestBody KpiStarSpecialRuleDTO dto, @CurrentUser LoginAuthBean user) {
        specialRuleBizService.applyKpiStarAdjustment(dto, user);
        return success();
    }

    /**
     * 绩效星级特殊调整人员
     *
     * @param user
     * @return
     */
    @GetMapping("/star-adjustment/staffs")
    @ControllerMethod("绩效星级特殊调整人员")
    public Message<List<StaffSelectVO>> queryLastMonthManageStaffs(@CurrentUser LoginAuthBean user) {
        return success(specialRuleBizService.queryLastMonthManageStaffs(user));
    }

    /**
     * 查询员工绩效星级
     *
     * @param user
     * @param yearMonth 月份，不传为当月
     * @param userId 不传查自己绩效星级
     * @return
     */
    @GetMapping("/staff-star")
    @ControllerMethod("查询员工绩效星级")
    public Message<String> getStaffKpiStar(@CurrentUser LoginAuthBean user, Long userId,
                                                        @RequestParam(required = false) YearMonth yearMonth) {
        if (PublicUtil.isEmpty(userId)) {
            userId = user.getUserId();
        }
        if (PublicUtil.isEmpty(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        return success(specialRuleBizService.getStaffKpiStar(userId, yearMonth));
    }

    /**
     * 绩效星级特殊调整详情
     *
     * @param approvalNo 审批单号
     * @return
     */
    @GetMapping("/star-adjustment")
    @ControllerMethod("绩效星级特殊调整详情")
    public Message<KpiStarSpecialRuleVO> getKpiStarAdjustment(@NotBlank(message = "审批单号不能为空") String approvalNo) {

        return success(specialRuleBizService.getKpiStarAdjustment(approvalNo));
    }

}
