package cn.fw.morax.server.controller.erp;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.kpi.KpiGroupRankDto;
import cn.fw.morax.domain.dto.query.*;
import cn.fw.morax.domain.vo.kpi.*;
import cn.fw.morax.service.biz.kpi.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 绩效排名配置
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@RequestMapping("/erp/kpi-rank")
@IgnoreAuth
public class KpiGroupRankController {

    private final KpiGroupRankBizService kpiGroupRankBizService;
    private final KpiGroupUserBizService kpiGroupUserBizService;

    /**
     * 绩效排名列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/page")
    @ControllerMethod("绩效排名列表")
    public Message<AppPage<KpiGroupRankVO>> kpiGroupRankPage(@Validated KpiGroupRankQueryDTO dto) {
        return success(kpiGroupRankBizService.kpiGroupRankPage(dto));
    }

    /**
     * 未选择绩效组列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/kpi-group/page")
    @ControllerMethod("未选择绩效组列表")
    public Message<AppPage<KpiGroupVO>> restKpiGroupPage(@Validated KpiGroupRankQueryDTO dto) {
        return success(kpiGroupRankBizService.restKpiGroupPage(dto));
    }

    /**
     * 绩效组排名配置保存
     *
     * @param dto
     * @return
     */
    @PostMapping("/config")
    @ControllerMethod("绩效组排名配置保存")
    public Message<Void> saveKpiGroupRank(@RequestBody @Validated KpiGroupRankDto dto, @CurrentUser LoginAuthBean currentUser) {
        kpiGroupRankBizService.saveKpiGroupRank(dto);
        return success();
    }


    /**
     * 绩效组排名组合详情
     *
     * @param id
     * @return
     */
    @GetMapping("/config")
    @ControllerMethod("绩效组排名组合详情")
    public Message<KpiGroupRankVO> kpiGroupRankDetail(@NotNull(message = "绩效组排名组合包id不能为空") Long id) {
        return success(kpiGroupRankBizService.kpiGroupRankDetail(id));
    }

    /**
     * 删除绩效组排名组合包
     *
     * @param id 绩效组排名组合包id
     * @return
     */
    @GetMapping("/del")
    @ControllerMethod("删除绩效组排名组合包")
    public Message<Void> removeById(@NotNull(message = "绩效组排名组合包id不能为空") Long id) {
        kpiGroupRankBizService.removeById(id);
        return success();
    }

    /**
     * 查询绩效组排名所有人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/all-staffs")
    @ControllerMethod("查询绩效组排名所有人员")
    public Message<List<KpiGroupUserVO>> queryRankStaffs(@Validated KpiGroupRankStaffQueryDTO dto) {
        return success(kpiGroupUserBizService.queryRankStaffs(dto.getKpiGroupIds()));
    }

    /**
     * 查询绩效组人员
     *
     * @param kpiGroupId
     * @return
     */
    @GetMapping("/group-staffs")
    @ControllerMethod("查询绩效组人员")
    public Message<List<KpiGroupUserVO>> queryKpiGroupStaffs(@NotNull(message = "绩效组id不能为空") Long kpiGroupId) {
        return success(kpiGroupUserBizService.queryKpiGroupStaffs(kpiGroupId));
    }

}
