package cn.fw.morax.server.controller.erp;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.salary.SalaryProjectAbleDTO;
import cn.fw.morax.domain.dto.salary.SalaryProjectDTO;
import cn.fw.morax.domain.dto.query.SalaryIndicatorQueryDTO;
import cn.fw.morax.domain.vo.salary.SalaryProjectVO;
import cn.fw.morax.service.biz.salary.SalaryProjectBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 薪酬指标配置
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@RequestMapping("/erp/salary/indicator")
public class SalaryIndicatorController {
    private final SalaryProjectBizService salaryProjectBizService;


    /**
     * 薪酬指标库列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/page")
    @ControllerMethod("薪酬指标库列表")
    public Message<AppPage<SalaryProjectVO>> salaryIndicatorsPage(@Validated SalaryIndicatorQueryDTO dto) {
        return success(salaryProjectBizService.salaryIndicatorsPage(dto));
    }

    /**
     * 薪酬指标保存
     *
     * @param dto 薪酬指标数据
     * @return
     */
    @PostMapping("/save")
    @ControllerMethod("薪酬指标保存")
    public Message<Void> saveSalaryIndicator(@Validated @RequestBody SalaryProjectDTO dto) {
        salaryProjectBizService.saveSalaryIndicator(dto);
        return success();
    }

    /**
     * 薪酬指标库禁用启用
     *
     * @return
     */
    @PutMapping("/enable")
    @ControllerMethod("薪酬指标库禁用启用")
    public Message<Void> enableIndicator(@Validated @RequestBody SalaryProjectAbleDTO dto) {
        salaryProjectBizService.enableIndicator(dto.getSalaryProjectId(), dto.getEnable());
        return success();
    }


}
