package cn.fw.morax.server.controller.erp;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.server.task.KpiCalcTask;
import cn.fw.morax.server.task.PayoffTask;
import cn.fw.morax.server.task.SalaryCalcTask;
import cn.fw.morax.server.task.SalaryGroupTask;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : DebugController
 * @description : 薪酬调试控制器
 * @date : 2022-04-11 10:45
 * @ignore
 */
@Slf4j
@RestController
@Authorization(AuthType.NONE)
@RequiredArgsConstructor
@Validated
@RequestMapping("/debug/task/salary")
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class SalaryTaskController {

    private final SalaryCalcTask salaryCalcTask;
    private final SalaryGroupTask salaryGroupTask;
    private final PayoffTask payoffTask;

    @GetMapping("/salary-sheet")
    @ControllerMethod("生成工资单")
    public Message<Void> createSalarySheet() {
        payoffTask.createSalarySheet();
        return success();
    }
    @GetMapping("/salary-confirm")
    @ControllerMethod("生成工资确认待办")
    public Message<Void> createSalaryConfirm() {
        payoffTask.createSalaryConfirmTodo();
        return success();
    }
    @GetMapping("/salary-confirm-todo")
    @ControllerMethod("推送工资确认待办")
    public Message<Void> createSalaryConfirmTodo() {
        payoffTask.pushTodo();
        return success();
    }

    @GetMapping("/salary-group")
    @ControllerMethod("需要计算薪酬的薪酬组写入缓存")
    public Message<Void> cacheCalculableKpiGroup() {
        salaryCalcTask.cacheCalculableKpiGroup();
        return success();
    }

    @GetMapping("/cal")
    @ControllerMethod("计算用户薪资")
    public Message<Void> calcUserSalary() {
        salaryCalcTask.calcUserSalary();
        return success();
    }

    @GetMapping("/star-adjustment")
    @ControllerMethod("星级申请变更后重新计算用户工资")
    public Message<Void> kpiChangeRetryCalcSalary() {
        salaryCalcTask.kpiChangeRetryCalcSalary();
        return success();
    }

    @GetMapping("/group-user")
    @ControllerMethod("薪酬组人员")
    public Message<Void> asyncSalaryGroupUser() {
        salaryGroupTask.asyncSalaryGroupUser();
        return success();
    }

}
