package cn.fw.morax.server.task;

import cn.fw.morax.rpc.oop.OopRpcService;
import cn.fw.morax.rpc.oop.dto.GroupDTO;
import cn.fw.morax.service.biz.kpi.KpiPoolCommonService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 通用定时器
 *
 * @author : kurisu
 * @version : 2.0
 * @className : GenericTask
 * @description : 通用定时器
 * @date : 2022-12-28 14:44
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class GenericTask {
    private final OopRpcService oopRpcService;
    private final KpiPoolCommonService kpiPoolCommonService;

    @Autowired
    public GenericTask(final OopRpcService oopRpcService,
                       final KpiPoolCommonService kpiPoolCommonService) {
        this.oopRpcService = oopRpcService;
        this.kpiPoolCommonService = kpiPoolCommonService;
    }

    /**
     * 固定绩效数据
     */
    @Scheduled(cron = "0 0 7 * * ?")
    public void regularKpiStar() {
        List<GroupDTO> list = oopRpcService.allGroups();
        if (!CollectionUtils.isEmpty(list)) {
            for (GroupDTO groupDTO : list) {
                kpiPoolCommonService.regularKpiStar(groupDTO.getId());
            }
        }
    }

    /**
     * 离职数据推送人事
     */
    @Scheduled(cron = "0 0/10 9-19 * * ?")
    public void forcedResign() {
        kpiPoolCommonService.forcedResign();
    }
}
