package cn.fw.morax.service.biz;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.ApprovalRecord;
import cn.fw.morax.domain.db.SettingDraft;
import cn.fw.morax.domain.dto.SettingDraftDTO;
import cn.fw.morax.domain.dto.eval.EvalDTO;
import cn.fw.morax.domain.dto.kpi.KpiGroupDTO;
import cn.fw.morax.domain.dto.query.KpiGroupQueryDTO;
import cn.fw.morax.domain.dto.query.SettingDraftQueryDTO;
import cn.fw.morax.domain.dto.salary.SalaryGroupDTO;
import cn.fw.morax.domain.enums.ApprovalTypeEnum;
import cn.fw.morax.domain.enums.SettingDraftStatusEnum;
import cn.fw.morax.domain.enums.SettingDraftTypeEnum;
import cn.fw.morax.domain.vo.SettingDraftVO;
import cn.fw.morax.rpc.approval.FlowApproveRpc;
import cn.fw.morax.service.data.ApprovalRecordService;
import cn.fw.morax.service.data.SettingDraftService;
import cn.fw.morax.service.data.kpi.KpiGroupService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author jiangchao
 * @des: 草稿功能
 * @date 2023/1/18 10:45
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class SettingDraftBizService {

    private final FlowApproveRpc flowApproveRpc;
    private final SettingDraftService settingDraftService;
    private final ApprovalRecordService approvalRecordService;


    /**
     * 分页查询
     * @param dto
     * @return
     */
    public AppPage<KpiGroupDTO> kpiGroupPage(SettingDraftQueryDTO dto) {
        PageData<SettingDraft> pageData = settingDraftService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<SettingDraft>lambdaQuery()
                        .eq(SettingDraft::getGroupId, dto.getGroupId())
                        .eq(SettingDraft::getType, dto.getType())
                        .eq(PublicUtil.isNotEmpty(dto.getPostId()), SettingDraft::getPostId, dto.getPostId())
                        .eq(PublicUtil.isNotEmpty(dto.getStatus()), SettingDraft::getStatus, dto.getStatus())
                        .eq(SettingDraft::getYn, Boolean.TRUE)
                        .last("ORDER BY id DESC")
        );

        Map<Long, String> finalApprovalRecordMap = getApprovalNo(pageData, dto.getType());
        return PublicUtil.toPage(pageData, settingDraft -> {
            KpiGroupDTO kpiGroupDTO = JSON.parseObject(settingDraft.getContent(), KpiGroupDTO.class);
            kpiGroupDTO.setApprovalNo(finalApprovalRecordMap.getOrDefault(settingDraft.getId(), ""));
            kpiGroupDTO.setDraftId(settingDraft.getId());
            kpiGroupDTO.setDraftStatus(settingDraft.getStatus());
            return kpiGroupDTO;
        });
    }

    /**
     * 分页查询
     * @param dto
     * @return
     */
    public AppPage<SalaryGroupDTO> salaryGroupPage(SettingDraftQueryDTO dto) {
        PageData<SettingDraft> pageData = settingDraftService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<SettingDraft>lambdaQuery()
                        .eq(SettingDraft::getGroupId, dto.getGroupId())
                        .eq(SettingDraft::getType, dto.getType())
                        .eq(PublicUtil.isNotEmpty(dto.getPostId()), SettingDraft::getPostId, dto.getPostId())
                        .eq(PublicUtil.isNotEmpty(dto.getStatus()), SettingDraft::getStatus, dto.getStatus())
                        .eq(SettingDraft::getYn, Boolean.TRUE)
                        .last("ORDER BY id DESC")
        );

        Map<Long, String> finalApprovalRecordMap = getApprovalNo(pageData, dto.getType());
        return PublicUtil.toPage(pageData, settingDraft -> {
            SalaryGroupDTO salaryGroupDTO = JSON.parseObject(settingDraft.getContent(), SalaryGroupDTO.class);
            salaryGroupDTO.setApprovalNo(finalApprovalRecordMap.getOrDefault(settingDraft.getId(), ""));
            salaryGroupDTO.setDraftId(settingDraft.getId());
            salaryGroupDTO.setDraftStatus(settingDraft.getStatus());
            return salaryGroupDTO;
        });
    }

    /**
     * 分页查询
     * @param dto
     * @return
     */
    public AppPage<EvalDTO> evalGroupPage(SettingDraftQueryDTO dto) {
        PageData<SettingDraft> pageData = settingDraftService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<SettingDraft>lambdaQuery()
                        .eq(SettingDraft::getGroupId, dto.getGroupId())
                        .eq(SettingDraft::getType, SettingDraftTypeEnum.EVAL)
                        .eq(SettingDraft::getStatus, dto.getStatus())
                        .eq(SettingDraft::getYn, Boolean.TRUE)
                        .last("ORDER BY id DESC")
        );

        Map<Long, String> finalApprovalRecordMap = getApprovalNo(pageData, dto.getType());
        return PublicUtil.toPage(pageData, settingDraft -> {
            EvalDTO evalDTO = JSON.parseObject(settingDraft.getContent(), EvalDTO.class);
            evalDTO.setApprovalNo(finalApprovalRecordMap.getOrDefault(settingDraft.getId(), ""));
            evalDTO.setDraftId(settingDraft.getId());
            evalDTO.setDraftStatus(settingDraft.getStatus());
            return evalDTO;
        });
    }

    /**
     * 获取分页查询的审批单号
     *
     * @param pageData
     * @param type
     * @return
     */
    public Map<Long, String> getApprovalNo(PageData<SettingDraft> pageData, Integer type) {
        if (PublicUtil.isEmpty(pageData.getRecords())) {
            return new HashMap<>();
        }
        ApprovalTypeEnum approvalType = ApprovalTypeEnum.KPI;
        if (SettingDraftTypeEnum.EVAL.getValue().equals(type)) {
            approvalType = ApprovalTypeEnum.EVAL;
        } else if(SettingDraftTypeEnum.SALARY.getValue().equals(type)) {
            approvalType = ApprovalTypeEnum.SALARY;
        }


        Map<Long, String> approvalRecordMap = new HashMap<>();
        if (PublicUtil.isNotEmpty(pageData.getRecords())) {
            List<Long> kpiGroupIds = pageData.getRecords().stream().map(SettingDraft::getId).collect(Collectors.toList());
            List<ApprovalRecord> approvalRecords = approvalRecordService.getLastApprovals(kpiGroupIds, approvalType);
            approvalRecordMap = approvalRecords.stream().collect(Collectors.toMap(ApprovalRecord::getDataId,
                    ApprovalRecord::getApprovalNo,
                    (v1, v2) -> {
                        log.error("同一dataId搜索出不同审批单号");
                        return v2;
                    }));
        }
        return approvalRecordMap;
    }

    /**
     * 保存绩效草稿
     *
     * @param dto
     */
    public void saveKpiDraft(SettingDraftDTO dto) {
        SettingDraft settingDraft = null;
        if (PublicUtil.isNotEmpty(dto.getId())) {
            settingDraft = settingDraftService.getById(dto.getId());
            BV.isTrue(PublicUtil.isNotEmpty(settingDraft), "考评草稿数据不存在");
        } else {
            settingDraft = new SettingDraft();
            settingDraft.setGroupId(dto.getGroupId());
            settingDraft.setYn(Boolean.TRUE);
            settingDraft.setType(SettingDraftTypeEnum.KPI);
        }

//        if (PublicUtil.isNotEmpty(dto.getKpiGroup().getId())) {
//            KpiGroup kpiGroup = kpiGroupService.getById(dto.getKpiGroup().getId());
//        }

        settingDraft.setContent(JSON.toJSONString(dto.getKpiGroup()));
        settingDraftService.saveOrUpdate(settingDraft);
    }
    /**
     * 保存薪酬草稿
     * @param dto
     */
    public void saveSalaryDraft(Long settingDraftId, SalaryGroupDTO dto, LoginAuthBean currentUser) {
        SettingDraft settingDraft = null;
        if (PublicUtil.isNotEmpty(settingDraftId)) {
            settingDraft = settingDraftService.getById(settingDraftId);
            BV.isTrue(PublicUtil.isNotEmpty(settingDraft), "考评草稿数据不存在");
        } else {
            settingDraft = new SettingDraft();
            settingDraft.setGroupId(currentUser.getGroupId());
            settingDraft.setUnionId(-1L);
            settingDraft.setType(SettingDraftTypeEnum.SALARY);
        }

        settingDraft.setContent(JSON.toJSONString(dto));
        settingDraftService.saveOrUpdate(settingDraft);
    }

    /**
     * 删除草稿
     *
     * @param id
     */
    public void delDraft(Long id, LoginAuthBean currentUser) {
        SettingDraft settingDraft = settingDraftService.getById(id);
        BV.notNull(settingDraft, "考评草稿数据不存在");
        if (SettingDraftStatusEnum.RELEASE_APPROVAL.equals(settingDraft.getStatus())) {
            throw new BusinessException("发布待审批 不能删除");
        }
        settingDraftService.update(Wrappers.<SettingDraft>lambdaUpdate()
                .set(SettingDraft::getUpdateTime, new Date())
                .set(SettingDraft::getYn, Boolean.FALSE)
                .eq(SettingDraft::getId, settingDraft.getId())
        );
    }

    /**
     * 撤销审批
     *
     * @param id
     */
    @Transactional
    public void cancelDraft(Long id, LoginAuthBean currentUser) {
        SettingDraft settingDraft = settingDraftService.getById(id);
        BV.notNull(settingDraft, "考评草稿数据不存在");
        if (! SettingDraftStatusEnum.RELEASE_APPROVAL.equals(settingDraft.getStatus())) {
            throw new BusinessException("发布待审批草稿才能撤销审批");
        }
        log.info("撤销审批,操作草稿 id，{}，操作人信息：{}", id, currentUser);
        settingDraft.setUpdateTime(new Date());
        settingDraft.setStatus(SettingDraftStatusEnum.RELEASE_APPROVAL_CANCEL);
        settingDraftService.updateById(settingDraft);

        ApprovalTypeEnum approvalType = ApprovalTypeEnum.KPI;
        if (SettingDraftTypeEnum.EVAL.equals(settingDraft.getType())) {
            approvalType = ApprovalTypeEnum.EVAL;
        } else if(SettingDraftTypeEnum.SALARY.equals(settingDraft.getType())) {
            approvalType = ApprovalTypeEnum.SALARY;
        }
        List<ApprovalRecord> approvalRecords = approvalRecordService.getLastApprovals(
                new ArrayList<Long>(){{add(settingDraft.getId());}},
                approvalType);
        if (PublicUtil.isEmpty(approvalRecords) || PublicUtil.isEmpty(approvalRecords.get(0).getApprovalNo())) {
            throw new BusinessException("撤销审批未找到审批单号");
        }
        flowApproveRpc.cancel(approvalRecords.get(0).getApprovalNo());
    }

    /**
     * 删除草稿
     *
     * @param id
     */
    public void delDraftByType(Long id, SettingDraftTypeEnum type, LoginAuthBean currentUser) {
        SettingDraft settingDraft = settingDraftService.getOne(Wrappers.<SettingDraft>lambdaQuery()
                        .eq(SettingDraft::getUnionId, id)
                        .eq(SettingDraft::getType, type)
                        .eq(SettingDraft::getYn, Boolean.TRUE)
                , Boolean.FALSE);
        if (PublicUtil.isEmpty(settingDraft)) {
            return;
        }
        settingDraft.setUpdateTime(new Date());
        settingDraft.setYn(Boolean.FALSE);
        settingDraftService.updateById(settingDraft);
    }

    /**
     * 绩效组详情
     *
     * @param id
     * @return
     */
    public KpiGroupDTO draftKpiGroupDetail(Long id) {
        SettingDraft settingDraft = this.getSettingDraft(id, SettingDraftTypeEnum.KPI);
        KpiGroupDTO kpiGroupDTO = JSON.parseObject(settingDraft.getContent(), KpiGroupDTO.class);
        kpiGroupDTO.setDraftId(settingDraft.getId());
        return kpiGroupDTO;
    }

    /**
     * 薪酬组详情
     *
     * @param id
     * @return
     */
    public SalaryGroupDTO draftSalaryGroupDetail(Long id) {
        SettingDraft settingDraft = this.getSettingDraft(id, SettingDraftTypeEnum.SALARY);
        SalaryGroupDTO salaryGroupDTO = JSON.parseObject(settingDraft.getContent(), SalaryGroupDTO.class);
        salaryGroupDTO.setDraftId(settingDraft.getId());
        return salaryGroupDTO;
    }

    /**
     * 考评组详情
     *
     * @param id
     * @return
     */
    public EvalDTO draftEvalDetail(Long id) {
        SettingDraft settingDraft = this.getSettingDraft(id, SettingDraftTypeEnum.EVAL);
        EvalDTO evalDTO = JSON.parseObject(settingDraft.getContent(), EvalDTO.class);
        evalDTO.setDraftId(settingDraft.getId());
        return evalDTO;
    }


    /**
     * 草稿详情
     *
     * @param id
     * @param type
     * @return
     */
    public SettingDraft getSettingDraft(Long id, SettingDraftTypeEnum type) {
        SettingDraft settingDraft = settingDraftService.getById(id);
        BV.notNull(settingDraft, "草稿配置不存在，请重试");
        return settingDraft;
    }
}
