package cn.fw.morax.service.biz.calculator.eval.kpi;

import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.bo.eval.EvalGroupUserShop;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.service.biz.calculator.Calculator;
import cn.fw.morax.service.data.eval.*;
import cn.fw.morax.service.data.kpi.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Consumer;

/**
 * 考评基础计算器
 *
 * @author : kurisu
 * @version : 2.0
 * @className : BaseCalculator
 * @description : 考评基础计算器
 * @date : 2022-12-26 14:17
 */
@Slf4j
public abstract class EvalKpiBaseCalculator implements Calculator<EvalGroupUserShop, EvalGroupIndicator, BigDecimal> {

    @Autowired
    protected EvalGroupIndicatorHitLogService evalGroupIndicatorHitLogService;
    @Autowired
    protected IndicatorUserValueService indicatorUserValueService;
    @Autowired
    protected EvalIndicatorValueService evalIndicatorValueService;
    @Autowired
    protected EvalGroupIndicatorParamService evalGroupIndicatorParamService;
    @Autowired
    protected EvalGroupIndicatorPreconditionService evalGroupIndicatorPreconditionService;
    @Autowired
    protected EvalGroupIndicatorPreconditionHitLogService preconditionHitLogService;
    @Autowired
    protected EvalGroupIndicatorPreconditionLaddersService preconditionLaddersService;
    @Autowired
    protected EvalGroupIndicatorTargetHitLogService evalGroupIndicatorTargetHitLogService;

    /**
     * 获取设置类型
     *
     * @return
     */
    public abstract ScoreWayEnum getCalMethod();

    /**
     * 通过前置条件计算绩效得分系数
     *
     * @param preconditions
     * @return
     */
    protected BigDecimal calcCoefficient(EvalGroupUserShop obj, List<EvalGroupIndicatorPrecondition> preconditions) {
        BigDecimal coefficient = BigDecimal.ONE;
        if (CollectionUtils.isEmpty(preconditions)) {
            return coefficient;
        }
        for (int i = 0; i < preconditions.size(); i++) {
            EvalGroupIndicatorPrecondition precondition = preconditions.get(i);
            TargetTypeEnum targetType = precondition.getTargetType();
            Optional<BigDecimal> valueOptional = queryValue(obj, precondition.getCodeType(), precondition.getCode());
            if (! valueOptional.isPresent()) {
                return BigDecimal.ZERO;
            }
            final BigDecimal originValue = valueOptional.get();//queryIndicatorValue(precondition.getCode(), valueOptional.get());
            BigDecimal indicatorValue = originValue;
            if (!TargetTypeEnum.NO.equals(targetType)) {
                BigDecimal targetValue = precondition.getTargetValue();
                TargetCalcTypeEnum targetCalcType = precondition.getTargetCalcType();
                if (TargetCalcTypeEnum.TARGET_VALUE.equals(targetCalcType)) {
                    indicatorValue = indicatorValue.divide(targetValue, 4, RoundingMode.HALF_UP);
                } else {
                    BigDecimal difference = BigDecimal.ONE.subtract(targetValue);
                    if (BigDecimal.ZERO.compareTo(difference) == 0) {
                        difference = BigDecimal.ONE;
                    }
                    BigDecimal userDiffValue = indicatorValue.subtract(targetValue);
                    if (userDiffValue.compareTo(BigDecimal.ZERO) <= 0) {
                        indicatorValue = BigDecimal.ZERO;
                    } else {
                        indicatorValue = userDiffValue.divide(difference, 4, RoundingMode.HALF_UP);
                    }
                }

                saveTargetHitLog(obj, precondition.getId(), IndicatorTypeEnum.PRE, originValue, indicatorValue);
            }
            coefficient = coefficient(obj, precondition.getId(), indicatorValue);
        }
        return coefficient;
    }

    protected BigDecimal coefficient(EvalGroupUserShop obj, Long preconditionId, BigDecimal value) {
        EvalGroupIndicatorPreconditionLadders ladder = preconditionLaddersService.getOne(Wrappers.<EvalGroupIndicatorPreconditionLadders>lambdaQuery()
                        .eq(EvalGroupIndicatorPreconditionLadders::getPreconditionId, preconditionId)
                        .gt(EvalGroupIndicatorPreconditionLadders::getUpper, value)
                        .le(EvalGroupIndicatorPreconditionLadders::getLower, value)
                        .eq(EvalGroupIndicatorPreconditionLadders::getYn, Boolean.TRUE),
                Boolean.FALSE
        );
        if (Objects.isNull(ladder)) {
            savePreconditionHitLog(obj, preconditionId, hitLog -> {
                hitLog.setPreconditionLaddersId(-1L);
                hitLog.setValue(value);
            });
            return BigDecimal.ZERO;
        }

        savePreconditionHitLog(obj, preconditionId, hitLog -> {
            hitLog.setPreconditionLaddersId(ladder.getId());
            hitLog.setValue(value);
        });
        return ladder.getScorePercent();
    }

    /**
     * 计算最终的指标值
     *
     * @param params
     * @param userShop
     * @param evalGroupIndicatorId
     * @return
     */
    protected BigDecimal calculateFinalIndValue(List<EvalGroupIndicatorParam> params, EvalGroupUserShop userShop, Long evalGroupIndicatorId) {
        BigDecimal rate = BigDecimal.ZERO;
        if (CollectionUtils.isEmpty(params)) {
            log.info("指标[{}]配置不存在", evalGroupIndicatorId);
            return rate;
        }
        for (EvalGroupIndicatorParam param : params) {
            Optional<BigDecimal> valueOptional = queryValue(userShop, param.getCodeType(), param.getCode());
            if (! valueOptional.isPresent()) {
                log.info("[{}] [{}]指标[{}]值不存在", userShop.getScopeType().getName(), userShop.getReferId(), evalGroupIndicatorId);
                continue;
            }
            TargetTypeEnum targetType = param.getTargetType();
            final BigDecimal originValue = valueOptional.get();
            BigDecimal indicatorValue = originValue;
            if (!TargetTypeEnum.NO.equals(targetType)) {
                BigDecimal targetValue = param.getTargetValue();
                TargetCalcTypeEnum targetCalcType = param.getTargetCalcType();
                if (TargetCalcTypeEnum.TARGET_VALUE.equals(targetCalcType)) {
                    indicatorValue = indicatorValue.divide(targetValue, 4, RoundingMode.HALF_UP);
                } else {
                    BigDecimal difference = BigDecimal.ONE.subtract(targetValue);
                    if (BigDecimal.ZERO.compareTo(difference) == 0) {
                        difference = BigDecimal.ONE;
                    }
                    BigDecimal userDiffValue = indicatorValue.subtract(targetValue);
                    if (userDiffValue.compareTo(BigDecimal.ZERO) <= 0) {
                        indicatorValue = BigDecimal.ZERO;
                    } else {
                        indicatorValue = userDiffValue.divide(difference, 4, RoundingMode.HALF_UP);
                    }
                }

                saveTargetHitLog(userShop, param.getId(), IndicatorTypeEnum.EXAMINE, originValue, indicatorValue);
            }
            boolean isCap = Boolean.TRUE.equals(param.getCap());
            BigDecimal proportion = param.getProportion();
            if (isCap) {
                indicatorValue = BigDecimal.ONE.compareTo(indicatorValue) > 0 ? indicatorValue : BigDecimal.ONE;
            }
            rate = rate.add(proportion.multiply(indicatorValue));
        }
        return rate;
    }

    /**
     * 查询指标值
     *
     * @param indicatorCode
     * @return
     */
    protected Optional<BigDecimal> queryValue(EvalGroupUserShop obj, IndicatorCodeTypeEnum codeType, String indicatorCode) {
        DimensionTypeEnum dimensionType = EvalScopeEnum.STAFF.equals(obj.getScopeType()) ? DimensionTypeEnum.STAFF : DimensionTypeEnum.SHOP;
        if (IndicatorCodeTypeEnum.INDICATOR.equals(codeType)) {
            LambdaQueryWrapper<IndicatorUserValue> queryWrapper = Wrappers.<IndicatorUserValue>lambdaQuery()
                    .eq(IndicatorUserValue::getIndicatorCode, indicatorCode)
                    .eq(IndicatorUserValue::getDataDate, obj.getDataDate())
                    .eq(IndicatorUserValue::getDimensionType, dimensionType)
                    .eq(IndicatorUserValue::getYn, Boolean.TRUE)
                    .eq(IndicatorUserValue::getGroupId, obj.getGroupId());
            if (EvalScopeEnum.STAFF.equals(obj.getScopeType())) {
                queryWrapper.eq(IndicatorUserValue::getUserId, obj.getReferId());
            } else {
                queryWrapper.eq(IndicatorUserValue::getShopId, obj.getReferId());
            }
            IndicatorUserValue indicatorUserValue = indicatorUserValueService.getOne(queryWrapper, Boolean.FALSE);
            return queryKpiIndicatorValue(indicatorUserValue);
        }

        EvalIndicatorValue evalIndicatorValue = evalIndicatorValueService.queryLastValue(obj.getReferId(),
                dimensionType, indicatorCode, obj.getDataDate());
        return Optional.ofNullable(evalIndicatorValue).map(EvalIndicatorValue::getIndicatorValue);
    }

    /**
     * 查询前置条件
     *
     * @param evalGroupIndicatorId
     * @return
     */
    protected List<EvalGroupIndicatorPrecondition> queryPrecondition(Long evalGroupIndicatorId) {
        List<EvalGroupIndicatorPrecondition> preList = evalGroupIndicatorPreconditionService.list(Wrappers.<EvalGroupIndicatorPrecondition>lambdaQuery()
                .eq(EvalGroupIndicatorPrecondition::getEvalGroupIndicatorId, evalGroupIndicatorId)
                .eq(EvalGroupIndicatorPrecondition::getYn, Boolean.TRUE)
                .orderByAsc(EvalGroupIndicatorPrecondition::getSort)
        );
        return Optional.ofNullable(preList).orElse(new ArrayList<>());
    }

    /**
     * 查询指标配置
     *
     * @param evalGroupIndicatorId
     * @return
     */
    protected List<EvalGroupIndicatorParam> queryParam(Long evalGroupIndicatorId) {
        List<EvalGroupIndicatorParam> list = evalGroupIndicatorParamService.list(Wrappers.<EvalGroupIndicatorParam>lambdaQuery()
                .eq(EvalGroupIndicatorParam::getEvalGroupIndicatorId, evalGroupIndicatorId)
                .eq(EvalGroupIndicatorParam::getYn, Boolean.TRUE)
        );
        return Optional.ofNullable(list).orElse(new ArrayList<>());
    }

    /**
     * 查询具体指标值
     *
     * @param indicatorUserValue
     * @return
     */
    protected Optional<BigDecimal> queryKpiIndicatorValue(IndicatorUserValue indicatorUserValue) {
        if (PublicUtil.isEmpty(indicatorUserValue) || PublicUtil.isEmpty(indicatorUserValue.getIndicatorValue())) {
            return Optional.empty();
        }
        BigDecimal value = BigDecimal.ZERO;
        try {
            JSONObject jsonObject = JSONObject.parseObject(indicatorUserValue.getIndicatorValue());
            value = Optional.ofNullable(jsonObject.getBigDecimal(indicatorUserValue.getIndicatorCode())).orElse(BigDecimal.ZERO);
        } catch (Exception e) {
            log.error("[{}]指标值转化失败", indicatorUserValue, e);
        }
        return Optional.of(value);
    }

    /**
     * 查询具体指标值
     *
     * @param indicatorCode
     * @param originalValue
     * @return
     */
    protected BigDecimal queryIndicatorValue(String indicatorCode, String originalValue) {
        BigDecimal value = BigDecimal.ZERO;
        try {
            JSONObject jsonObject = JSONObject.parseObject(originalValue);
            value = Optional.ofNullable(jsonObject.getBigDecimal(indicatorCode)).orElse(BigDecimal.ZERO);
        } catch (Exception e) {
            log.error("[{}]指标值转化失败", indicatorCode, e);
        }
        return value;
    }

    protected void savePreconditionHitLog(EvalGroupUserShop userShop, Long preconditionId, Consumer<EvalGroupIndicatorPreconditionHitLog> consumer) {
        Long poolId = userShop.getPoolId();
        EvalScopeEnum evalScope = userShop.getScopeType();

        preconditionHitLogService.remove(Wrappers.<EvalGroupIndicatorPreconditionHitLog>lambdaQuery()
                .eq(EvalGroupIndicatorPreconditionHitLog::getPoolId, poolId)
                .eq(EvalGroupIndicatorPreconditionHitLog::getPreconditionId, preconditionId)
                .eq(EvalGroupIndicatorPreconditionHitLog::getDataDate, userShop.getDataDate())
                .eq(EvalGroupIndicatorPreconditionHitLog::getGroupId, userShop.getGroupId())
        );
        EvalGroupIndicatorPreconditionHitLog hitLog = new EvalGroupIndicatorPreconditionHitLog();
        hitLog.setPreconditionId(preconditionId);
        hitLog.setPoolId(poolId);
        hitLog.setScopeType(evalScope);
        hitLog.setDataDate(userShop.getDataDate());
        hitLog.setGroupId(userShop.getGroupId());
        if (consumer != null) {
            consumer.accept(hitLog);
        }
        preconditionHitLogService.save(hitLog);
    }

    protected void saveTargetHitLog(EvalGroupUserShop userShop, Long referId, IndicatorTypeEnum targetType, BigDecimal value, BigDecimal reachValue) {
        evalGroupIndicatorTargetHitLogService.remove(Wrappers.<EvalGroupIndicatorTargetHitLog>lambdaQuery()
                .eq(EvalGroupIndicatorTargetHitLog::getPoolId, userShop.getPoolId())
                .eq(EvalGroupIndicatorTargetHitLog::getScopeType, userShop.getScopeType())
                .eq(EvalGroupIndicatorTargetHitLog::getTargetType, targetType)
                .eq(EvalGroupIndicatorTargetHitLog::getDataDate, userShop.getDataDate())
                .eq(EvalGroupIndicatorTargetHitLog::getReferId, referId)
        );
        EvalGroupIndicatorTargetHitLog log = new EvalGroupIndicatorTargetHitLog();
        log.setPoolId(userShop.getPoolId());
        log.setScopeType(userShop.getScopeType());
        log.setReferId(referId);
        log.setTargetType(targetType);
        log.setValue(value);
        log.setReachValue(reachValue);
        log.setDataDate(userShop.getDataDate());
        log.setGroupId(userShop.getGroupId());
        log.setYn(Boolean.TRUE);
        evalGroupIndicatorTargetHitLogService.save(log);
    }

    /**
     * 储存命中记录
     *
     * @param evalGroupIndicatorId
     * @param userShop
     */
    protected void saveParamHitLog(Long evalGroupIndicatorId, EvalGroupUserShop userShop,
                                   Consumer<EvalGroupIndicatorHitLog> consumer) {
        EvalGroupIndicatorHitLog hitLog = new EvalGroupIndicatorHitLog();
        hitLog.setEvalGroupIndicatorId(evalGroupIndicatorId);
        hitLog.setPoolId(userShop.getPoolId());
        hitLog.setScopeType(userShop.getScopeType());
        hitLog.setDataDate(userShop.getDataDate());
        hitLog.setGroupId(userShop.getGroupId());
        hitLog.setYn(Boolean.TRUE);
        if (consumer != null) {
            consumer.accept(hitLog);
        }
        evalGroupIndicatorHitLogService.saveUnique(hitLog);
    }

}
