package cn.fw.morax.service.biz.calculator.eval.kpi;

import cn.fw.morax.domain.bo.eval.EvalGroupUserShop;
import cn.fw.morax.domain.db.eval.EvalGroupIndicator;
import cn.fw.morax.domain.db.eval.EvalGroupIndicatorLadders;
import cn.fw.morax.domain.db.eval.EvalGroupIndicatorParam;
import cn.fw.morax.domain.db.eval.EvalGroupIndicatorPrecondition;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import cn.fw.morax.service.data.eval.EvalGroupIndicatorLaddersService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Objects;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : LadderCalculator
 * @description : 阶梯计算器
 * @date : 2022-04-18 11:47
 */
@Component
@Slf4j
public class EvalLadderCalculator extends EvalKpiBaseCalculator {
    private final EvalGroupIndicatorLaddersService evalGroupIndicatorLaddersService;

    @Autowired
    public EvalLadderCalculator(final EvalGroupIndicatorLaddersService evalGroupIndicatorLaddersService) {
        this.evalGroupIndicatorLaddersService = evalGroupIndicatorLaddersService;
    }

    @Override
    public ScoreWayEnum getCalMethod() {
        return ScoreWayEnum.LADDER;
    }

    @Override
    public BigDecimal calculate(EvalGroupUserShop obj, EvalGroupIndicator bo) {
        final BigDecimal baseScore = BigDecimal.valueOf(bo.getBaseScore());
        final Long evalGroupIndicatorId = bo.getId();
        List<EvalGroupIndicatorPrecondition> preconditions = queryPrecondition(evalGroupIndicatorId);
        BigDecimal coefficient = calcCoefficient(obj, preconditions);
        return calcScore(evalGroupIndicatorId, baseScore, coefficient, obj);
    }

    /**
     * 通过配置条件计算绩效分
     *
     * @return
     */
    private BigDecimal calcScore(Long evalGroupIndicatorId, BigDecimal baseScore, BigDecimal coefficient, EvalGroupUserShop obj) {
        List<EvalGroupIndicatorParam> paramList = queryParam(evalGroupIndicatorId);
        BigDecimal finalIndValue = calculateFinalIndValue(paramList, obj, evalGroupIndicatorId);
        BigDecimal score = ladderScore(evalGroupIndicatorId, finalIndValue, obj);
        if (baseScore.compareTo(score) < 0) {
            score = baseScore;
        }
        return score.multiply(coefficient).divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);
    }

    private BigDecimal ladderScore(Long kpiGroupIndicatorId, BigDecimal rate, EvalGroupUserShop obj) {
        EvalGroupIndicatorLadders ladder = evalGroupIndicatorLaddersService.getOne(Wrappers.<EvalGroupIndicatorLadders>lambdaQuery()
                        .eq(EvalGroupIndicatorLadders::getEvalGroupIndicatorId, kpiGroupIndicatorId)
                        .eq(EvalGroupIndicatorLadders::getYn, Boolean.TRUE)
                        .gt(EvalGroupIndicatorLadders::getUpper, rate)
                        .le(EvalGroupIndicatorLadders::getLower, rate),
                Boolean.FALSE
        );
        if (Objects.isNull(ladder)) {
            return BigDecimal.ZERO;
        }
        saveParamHitLog(kpiGroupIndicatorId, obj, hitLog -> {
            hitLog.setEvalGroupIndicatorLaddersId(ladder.getId());
            hitLog.setHitLadderValue(rate);
            hitLog.setHitCommissionValue(BigDecimal.ZERO);
        });
        BigDecimal standardScore = BigDecimal.valueOf(ladder.getStandardScore());
        return standardScore.multiply(rate);
    }
}
