package cn.fw.morax.service.biz.calculator.eval.reward;

import cn.fw.morax.domain.bo.eval.EvalGroupUserShop;
import cn.fw.morax.domain.db.eval.EvalGroupReward;
import cn.fw.morax.domain.db.eval.EvalGroupRewardParam;
import cn.fw.morax.domain.db.eval.EvalGroupRewardPrecondition;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.ParamTypeEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * @author jiangchao
 * @des: 固定
 * @date 2023/2/1 11:03
 */
@Component
@Slf4j
public class EvalRewardFixationCalculator extends EvalRewardBaseCalculator {

    @Override
    public EvalRewardCalMethodEnum getCalMethod() {
        return EvalRewardCalMethodEnum.FIXATION;
    }

    @Override
    public BigDecimal calculate(EvalGroupUserShop userShop, EvalGroupReward reward) {
        final Long evalGroupRewardId = reward.getId();
        List<EvalGroupRewardPrecondition> preconditions = queryPrecondition(evalGroupRewardId);
        boolean examined = examinePrecondition(userShop, preconditions);

        BigDecimal stairValue = reward.getMoney();
        List<EvalGroupRewardParam> params = queryProjectParam(evalGroupRewardId, ParamTypeEnum.COMMISSION);
        if (CollectionUtils.isEmpty(params)) {
            return stairValue;
        }

        final BigDecimal commissionValue = calculateParamValue(userShop, params);
        if (!examined) {
            return BigDecimal.ZERO;
        }
        final BigDecimal salaryValue = stairValue.multiply(commissionValue).divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);

        clearProjectHitLog(evalGroupRewardId, userShop);
        saveProjectHitLog(evalGroupRewardId, userShop, log -> {
            log.setRewardValue(salaryValue);
            log.setHitCommissionValue(commissionValue);
        });
        return salaryValue;
    }

}
