package cn.fw.morax.service.biz.calculator.eval.reward;

import cn.fw.morax.domain.bo.eval.EvalGroupUserShop;
import cn.fw.morax.domain.db.eval.EvalGroupReward;
import cn.fw.morax.domain.db.eval.EvalGroupRewardLadders;
import cn.fw.morax.domain.db.eval.EvalGroupRewardParam;
import cn.fw.morax.domain.db.eval.EvalGroupRewardPrecondition;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.IndicatorTypeEnum;
import cn.fw.morax.domain.enums.ParamTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @author jiangchao
 * @des: 阶梯
 * @date 2023/2/1 11:03
 */
@Component
@Slf4j
public class EvalRewardLadderCalculator extends EvalRewardBaseCalculator {

    @Override
    public EvalRewardCalMethodEnum getCalMethod() {
        return EvalRewardCalMethodEnum.LADDER;
    }

    @Override
    public BigDecimal calculate(EvalGroupUserShop userShop, EvalGroupReward reward) {
        final Long evalGroupRewardId = reward.getId();
        List<EvalGroupRewardLadders> ladders = queryRewardLadders(evalGroupRewardId);
        if (CollectionUtils.isEmpty(ladders)) {
            log.error("[{}]考评阶梯配置不存在", evalGroupRewardId);
            return BigDecimal.ZERO;
        }
        List<EvalGroupRewardPrecondition> preconditions = queryPrecondition(evalGroupRewardId);
        boolean examined = examinePrecondition(userShop, preconditions);

        List<EvalGroupRewardParam> ladderParams = queryProjectParam(evalGroupRewardId, ParamTypeEnum.LADDER);
        if (CollectionUtils.isEmpty(ladderParams)) {
            log.error("配置id[{}]的台阶项不存在", evalGroupRewardId);
            return BigDecimal.ZERO;
        }
        final BigDecimal ladderValue = calculateParamValue(userShop, ladderParams);
        EvalGroupRewardLadders ladder = ladders.stream()
                .filter(r -> ladderValue.compareTo(r.getLower()) >= 0)
                .filter(r -> ladderValue.compareTo(r.getUpper()) < 0)
                .findFirst()
                .orElse(null);
        if (Objects.isNull(ladder)) {
            log.error("[{}]，referId: [{}]指标配置id[{}]为匹配到任何台阶", userShop.getScopeType().getName(), userShop.getReferId(), evalGroupRewardId);
            return BigDecimal.ZERO;
        }
        BigDecimal money = ladder.getMoney();
        BigDecimal commissionValue = null;
        BigDecimal rewardValue = null;
        List<EvalGroupRewardParam> rewardParams = queryProjectParam(evalGroupRewardId, ParamTypeEnum.COMMISSION);
        commissionValue = calculateParamValue(userShop, rewardParams);

        if (!examined) {
            return BigDecimal.ZERO;
        }
        rewardValue = money.multiply(commissionValue).divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);

        final BigDecimal _rewardValue = rewardValue;
        final BigDecimal _commissionValue = commissionValue;
        clearProjectHitLog(evalGroupRewardId, userShop);
        saveProjectHitLog(evalGroupRewardId, userShop, log -> {
            log.setEvalGroupRewardLaddersId(ladder.getId());
            log.setRewardValue(_rewardValue);
            log.setHitLadderValue(ladderValue);
            log.setHitCommissionValue(_commissionValue);
        });
        return _rewardValue;
    }


    /**
     * 计算用户最终值
     *
     * @return
     */
    protected BigDecimal calculateParamValue(EvalGroupUserShop param, List<EvalGroupRewardParam> rewardParams) {
        BigDecimal commissionValue = BigDecimal.ZERO;
        for (EvalGroupRewardParam projectParam : rewardParams) {
            String indicatorCode = projectParam.getCode();
            boolean isCap = Boolean.TRUE.equals(projectParam.getCap());
            BigDecimal proportion = projectParam.getProportion();
            TargetTypeEnum targetType = projectParam.getTargetType();
            ParamTypeEnum paramType = projectParam.getParamType();
            final BigDecimal userOriginValue = queryValue(param, projectParam.getCodeType(), indicatorCode).orElse(BigDecimal.ZERO);
            BigDecimal _calcValue = userOriginValue;
            if (!TargetTypeEnum.NO.equals(targetType)) {
                _calcValue = calculateTargetValue(projectParam.getTargetCalcType(), projectParam.getTargetValue(), _calcValue);
                saveTargetHitLog(param,projectParam.getId(), IndicatorTypeEnum.EXAMINE, userOriginValue, _calcValue);
            }
            if (isCap) {
                _calcValue = _calcValue.compareTo(BigDecimal.ONE) > 0 ? BigDecimal.ONE : _calcValue;
            }
            commissionValue = commissionValue.add(proportion.multiply(_calcValue));
        }
        return commissionValue;
    }

}
