package cn.fw.morax.service.biz.calculator.kpi;

import cn.fw.morax.domain.bo.kpi.KpiIndicatorBO;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorParam;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorPrecondition;
import cn.fw.morax.domain.db.kpi.KpiGroupUser;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : NormalCalculator
 * @description : 通用计算器
 * @date : 2022-04-18 11:46
 */
@Component
@Slf4j
public class NormalCalculator extends KpiAbsBaseCalculator {
    @Override
    public ScoreWayEnum getCalMethod() {
        return ScoreWayEnum.NORMAL;
    }

    @Override
    public BigDecimal calculate(KpiGroupUser user, KpiIndicatorBO bo) {
        final BigDecimal baseScore = BigDecimal.valueOf(bo.getBaseScore());
        final Long kpiGroupIndicatorId = bo.getKpiGroupIndicatorId();
        List<KpiGroupIndicatorPrecondition> list = queryPrecondition(kpiGroupIndicatorId, bo.getKpiGroupId());
        BigDecimal coefficient = calcCoefficient(user, list);
        List<KpiGroupIndicatorParam> paramList = queryParam(kpiGroupIndicatorId, bo.getKpiGroupId());
        BigDecimal finalIndValue = calculateFinalIndValue(paramList, user, kpiGroupIndicatorId);
        BigDecimal multiply = baseScore.multiply(finalIndValue).multiply(coefficient);
        return multiply.divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);
    }
}
