package cn.fw.morax.service.biz.calculator.salary;

import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.enums.ParamTypeEnum;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : ConditionSalaryCalculator
 * @description : 条件薪资计算器 目前适用与车系提成计算
 * @date : 2022-04-28 14:34
 */
@Component
@Slf4j
public class ConditionSalaryCalculator extends SalaryBaseCalculator {
    @Override
    public BigDecimal calculate(SalaryGroupProject param1, SalaryGroupUser param2) {
        if (Boolean.TRUE.equals(param2.getProbationer())) {
            return null;
        }
        final Long salaryGroupProjectId = param1.getId();
        List<SalaryGroupProjectSettin> settins = queryProjectSettins(salaryGroupProjectId);
        if (CollectionUtils.isEmpty(settins)) {
            log.error("[{}]薪酬项配置不存在", salaryGroupProjectId);
            return BigDecimal.ZERO;
        }
        List<SalaryGroupProjectPrecondition> preconditionList = queryPrecondition(salaryGroupProjectId);
        boolean examined = examinePrecondition(preconditionList, param2);
        if (!examined) {
            return BigDecimal.ZERO;
        }
        Map<Long, SalaryGroupProjectSettin> map = settins.stream().collect(Collectors.toMap(SalaryGroupProjectSettin::getStairKey, a -> a));
        BigDecimal totalStairValue = BigDecimal.ZERO;
        List<SalaryGroupProjectParam> projectParamList = queryProjectParam(salaryGroupProjectId, ParamTypeEnum.COMMISSION);
        if (CollectionUtils.isEmpty(projectParamList)) {
            log.info("指标配置id[{}]的提成计算项不存在", salaryGroupProjectId);
            return BigDecimal.ZERO;
        }
        SalaryGroupProjectParam projectParam = projectParamList.get(0);
        String indicatorCode = projectParam.getIndicatorCode();
        JSONObject jsonObject = queryUserIndicatorValue(param2.getUserId(), param2.getDataDate(), indicatorCode);
        clearProjectHitLog(salaryGroupProjectId, param2);
        for (Map.Entry<Long, SalaryGroupProjectSettin> entry : map.entrySet()) {
            Long key = entry.getKey();
            SalaryGroupProjectSettin settin = entry.getValue();
            BigDecimal value = settin.getStairValue();
            BigDecimal userValue = Optional.ofNullable(jsonObject.getBigDecimal(String.valueOf(key))).orElse(BigDecimal.ZERO);
            BigDecimal stairValue = userValue.multiply(value);
            totalStairValue = stairValue.add(totalStairValue);
            saveProjectHitLog(salaryGroupProjectId, param2, log -> {
                log.setSalaryGroupProjectSettinId(settin.getId());
                log.setSalaryValue(stairValue);
                log.setHitCommissionValue(userValue);
            });
        }
        return totalStairValue;
    }

    @Override
    public SalaryCalMethodEnum getCalMethod() {
        return SalaryCalMethodEnum.CAR_SERIES;
    }
}
