package cn.fw.morax.service.biz.calculator.salary;

import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.enums.ParamTypeEnum;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : FixattionSalaryCalculator
 * @description : 固定金额计算器
 * @date : 2022-04-27 10:34
 */
@Component
@Slf4j
public class FixattionSalaryCalculator extends SalaryBaseCalculator {
    @Override
    public BigDecimal calculate(SalaryGroupProject salaryGroupProject, SalaryGroupUser user) {
        if (Boolean.TRUE.equals(user.getProbationer())) {
            return null;
        }
        final Long salaryGroupProjectId = salaryGroupProject.getId();
        SalaryGroupProjectSettin projectSettin = queryProjectSettins(salaryGroupProjectId).stream().findAny().orElse(null);
        if (Objects.isNull(projectSettin)) {
            log.error("[{}]薪酬项配置不存在", salaryGroupProjectId);
            return BigDecimal.ZERO;
        }
        List<SalaryGroupProjectPrecondition> preconditionList = queryPrecondition(salaryGroupProjectId);
        boolean examined = examinePrecondition(preconditionList, user);
        if (!examined) {
            return BigDecimal.ZERO;
        }
        BigDecimal stairValue = projectSettin.getStairValue();
        List<SalaryGroupProjectParam> projectParamList = queryProjectParam(salaryGroupProjectId, ParamTypeEnum.COMMISSION);
        if (CollectionUtils.isEmpty(projectParamList)) {
            return stairValue;
        }
        final BigDecimal commissionValue = calculateParamValue(projectParamList, user);
        final BigDecimal salaryValue = stairValue.multiply(commissionValue).divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);
        clearProjectHitLog(salaryGroupProjectId, user);
        saveProjectHitLog(salaryGroupProjectId, user, log -> {
            log.setSalaryGroupProjectSettinId(projectSettin.getId());
            log.setSalaryValue(salaryValue);
            log.setHitLadderValue(null);
            log.setHitCommissionValue(commissionValue);
        });
        return salaryValue;

    }

    @Override
    public SalaryCalMethodEnum getCalMethod() {
        return SalaryCalMethodEnum.FIXATION;
    }
}
