package cn.fw.morax.service.biz.eval;

import cn.fw.backlog.sdk.api.result.FailBacklogItem;
import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.morax.common.config.FlowVal;
import cn.fw.morax.common.config.TodoVal;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.ThreadPoolUtil;
import cn.fw.morax.domain.bo.eval.EvalGroupImportBO;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.domain.vo.eval.EvalIndicatorImportUserVO;
import cn.fw.morax.domain.vo.eval.EvalReportVO;
import cn.fw.morax.domain.vo.eval.EvalShopVO;
import cn.fw.morax.rpc.approval.FlowApproveRpc;
import cn.fw.morax.rpc.backlog.TodoRpcService;
import cn.fw.morax.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.morax.rpc.backlog.dto.BacklogBatchPlanItemReqDTO;
import cn.fw.morax.rpc.backlog.dto.BacklogBatchPlanReqDTO;
import cn.fw.morax.rpc.erp.ErpRpcService;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleInfoDTO;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleShopDTO;
import cn.fw.morax.rpc.oop.OopRpcService;
import cn.fw.morax.rpc.oop.dto.ShopDTO;
import cn.fw.morax.service.biz.ApprovalBizService;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.morax.service.data.ApprovalRecordService;
import cn.fw.morax.service.data.SettingDraftService;
import cn.fw.morax.service.data.eval.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.locks.Lock;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author jiangchao
 * @des: 考评指标数据
 * @date 2023/2/3 15:31
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class EvalIndicatorValueBizService {

    private final EvalGroupIndicatorPreconditionService evalGroupIndicatorPreconditionService;
    private final EvalGroupRewardPreconditionService evalGroupRewardPreconditionService;
    private final EvalGroupIndicatorParamService evalGroupIndicatorParamService;
    private final EvalGroupRewardParamService evalGroupRewardParamService;
    private final EvalIndicatorService evalIndicatorService;
    private final EvalGroupUserBizService evalGroupUserBizService;
    private final StringRedisTemplate stringRedisTemplate;
    private final EvalGroupService evalGroupService;
    private final EvalGroupUserService evalGroupUserService;
    private final EvalIndicatorValueService evalIndicatorValueService;
    private final EvalIndicatorImportService evalIndicatorImportService;
    private final DistributedLocker distributedLocker;
    private final ErpRpcService erpRpcService;
    private final TodoRpcService todoRpcService;
    private final OopRpcService oopRpcService;
    private final TodoVal todoVal;


    @Value("${spring.cache.custom.global-prefix}:import:eval-indicator")
    @Getter
    private String evalImportIndicator;
    @Value("${spring.cache.custom.global-prefix}:push:import-todo")
    @Getter
    private String pushImportLockKey;
    @Value("${spring.cache.custom.global-prefix}:check:import-todo")
    @Getter
    private String checkTodoLockKey;

    public void cacheReportEvalIndicator(LocalDate now) {
        List<EvalGroup> evalGroups = evalGroupService.getAllEffectGroups(now);
        if (PublicUtil.isEmpty(evalGroups)) {
            return;
        }
        Map<Long, List<EvalGroup>> evalGroupMap = evalGroups.stream().collect(Collectors.groupingBy(EvalGroup::getGroupId));

        List<EvalGroupImportBO> evalGroupImportBOS = new ArrayList();
        for (Map.Entry<Long, List<EvalGroup>> entry : evalGroupMap.entrySet()) {
            Long groupId = entry.getKey();
            List<Long> evalGroupIds = entry.getValue().stream().map(EvalGroup::getId).collect(Collectors.toList());
            Map<String, Set<Long>> indicatorCodeEvalGroupMap = getIndicatorEvalGroupIds(evalGroupIds);

            if (PublicUtil.isEmpty(indicatorCodeEvalGroupMap)) {
                continue;
            }
            Set<String> codes = indicatorCodeEvalGroupMap.keySet();
            List<EvalIndicator> evalIndicators = evalIndicatorService.list(Wrappers.<EvalIndicator>lambdaQuery()
                    .eq(EvalIndicator::getYn, Boolean.TRUE)
                    .in(EvalIndicator::getCode, codes)
            );
            //需要导入数据的指标编码
            Set<String> needImportCodes = evalIndicators.stream()
                    .filter(indicator -> checkUpdateCycle(indicator, now))
                    .map(EvalIndicator::getCode).collect(Collectors.toSet());

            Map<String, EvalIndicator> codeIndicatorMap = evalIndicators.stream().collect(
                    Collectors.toMap(EvalIndicator::getCode, Function.identity(), (v1, v2) -> v1));

            //编码、考评组id
            for (Map.Entry<String, Set<Long>> evalGroupIndicatorCode : indicatorCodeEvalGroupMap.entrySet()) {
                if (needImportCodes.contains(evalGroupIndicatorCode.getKey())) {
                    LocalDateTime importExpireTime = getNextTime(codeIndicatorMap.get(evalGroupIndicatorCode.getKey()), now);
                    evalGroupImportBOS.add(new EvalGroupImportBO(groupId, evalGroupIndicatorCode.getKey(),
                            importExpireTime, evalGroupIndicatorCode.getValue()));
                }
            }
        }


        //需要导入数据的指标
        BoundListOperations<String, String> listOps = stringRedisTemplate.boundListOps(getEvalImportIndicator());
        if (PublicUtil.isNotEmpty(evalGroupImportBOS)) {
            for (EvalGroupImportBO evalGroupImportBO : evalGroupImportBOS) {
                listOps.rightPush(JSONObject.toJSONString(evalGroupImportBO));
            }
        }

    }

    /**
     * 获取指标对应生效中的考评组id
     *
     * @param evalGroupIds
     * @return
     */
    public Map<String, Set<Long>> getIndicatorEvalGroupIds(List<Long> evalGroupIds) {

        Map<String, Set<Long>> indicatorCodeEvalGroupMap = new HashMap<>();

        List<EvalGroupIndicatorParam> indicatorParams = evalGroupIndicatorParamService.list(Wrappers.<EvalGroupIndicatorParam>lambdaQuery()
                .in(EvalGroupIndicatorParam::getEvalGroupId, evalGroupIds)
                .eq(EvalGroupIndicatorParam::getCodeType, IndicatorCodeTypeEnum.EVAL_INDICATOR)
                .eq(EvalGroupIndicatorParam::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isNotEmpty(indicatorParams)) {
            for (EvalGroupIndicatorParam indicatorParam : indicatorParams) {
                Set<Long> codeEvalGroupIds = indicatorCodeEvalGroupMap.getOrDefault(indicatorParam.getCode(), new HashSet<>());
                codeEvalGroupIds.add(indicatorParam.getEvalGroupId());
                indicatorCodeEvalGroupMap.put(indicatorParam.getCode(), codeEvalGroupIds);
            }
        }

        List<EvalGroupIndicatorPrecondition> indicatorPreconditions = evalGroupIndicatorPreconditionService.list(Wrappers.<EvalGroupIndicatorPrecondition>lambdaQuery()
                .eq(EvalGroupIndicatorPrecondition::getEvalGroupId, evalGroupIds)
                .eq(EvalGroupIndicatorPrecondition::getCodeType, IndicatorCodeTypeEnum.EVAL_INDICATOR)
                .eq(EvalGroupIndicatorPrecondition::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isNotEmpty(indicatorPreconditions)) {
            for (EvalGroupIndicatorPrecondition indicatorPrecondition : indicatorPreconditions) {
                Set<Long> codeEvalGroupIds = indicatorCodeEvalGroupMap.getOrDefault(indicatorPrecondition.getCode(), new HashSet<>());
                codeEvalGroupIds.add(indicatorPrecondition.getEvalGroupId());
                indicatorCodeEvalGroupMap.put(indicatorPrecondition.getCode(), codeEvalGroupIds);
            }
        }

        List<EvalGroupRewardParam> rewardParams = evalGroupRewardParamService.list(Wrappers.<EvalGroupRewardParam>lambdaQuery()
                .in(EvalGroupRewardParam::getEvalGroupId, evalGroupIds)
                .eq(EvalGroupRewardParam::getCodeType, IndicatorCodeTypeEnum.EVAL_INDICATOR)
                .eq(EvalGroupRewardParam::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isNotEmpty(rewardParams)) {
            for (EvalGroupRewardParam rewardParam : rewardParams) {
                Set<Long> codeEvalGroupIds = indicatorCodeEvalGroupMap.getOrDefault(rewardParam.getCode(), new HashSet<>());
                codeEvalGroupIds.add(rewardParam.getEvalGroupId());
                indicatorCodeEvalGroupMap.put(rewardParam.getCode(), codeEvalGroupIds);
            }
        }
        List<EvalGroupRewardPrecondition> rewardPreconditions = evalGroupRewardPreconditionService.list(Wrappers.<EvalGroupRewardPrecondition>lambdaQuery()
                .in(EvalGroupRewardPrecondition::getEvalGroupId, evalGroupIds)
                .eq(EvalGroupRewardPrecondition::getCodeType, IndicatorCodeTypeEnum.EVAL_INDICATOR)
                .eq(EvalGroupRewardPrecondition::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isNotEmpty(rewardPreconditions)) {
            for (EvalGroupRewardPrecondition rewardPrecondition : rewardPreconditions) {
                Set<Long> codeEvalGroupIds = indicatorCodeEvalGroupMap.getOrDefault(rewardPrecondition.getCode(), new HashSet<>());
                codeEvalGroupIds.add(rewardPrecondition.getEvalGroupId());
                indicatorCodeEvalGroupMap.put(rewardPrecondition.getCode(), codeEvalGroupIds);
            }
        }
        return indicatorCodeEvalGroupMap;
    }

    public static void main(String[] args) {
        LocalDate now = LocalDate.now();
        EvalIndicator day = new EvalIndicator();
        EvalIndicator week = new EvalIndicator();
        EvalIndicator fixDay = new EvalIndicator();
        EvalIndicator postRollMonth = new EvalIndicator();
        EvalIndicator countRollMonth = new EvalIndicator();

//        //频率类型; 1:按滚动天 2:按星期 3:按月指定日期 4：按滚动月
//        day.setRuleType(EvalRuleTypeEnum.DAY);
//        day.setStartTime(LocalDate.of(2023, 2, 1));
//        day.setRollValue(5);
//        System.out.println("day  " + checkUpdateCycle(day, now));
//
//        week.setRuleType(EvalRuleTypeEnum.WEEK_DAY);
//        week.setRuleValues(new ArrayList<Integer>(){{add(1);add(2);add(4);}});
//        System.out.println("week  " + checkUpdateCycle(week, now));
//
//        fixDay.setRuleType(EvalRuleTypeEnum.MONTH_FIX_DAY);
//        fixDay.setRuleValues(new ArrayList<Integer>(){{add(8);add(16);}});
//        System.out.println("fixDay  " + checkUpdateCycle(fixDay, now));
//
//        postRollMonth.setRuleType(EvalRuleTypeEnum.MONTH);
//        postRollMonth.setOrderType(RollMonthOrderTypeEnum.POSITIVE);
//        postRollMonth.setStartTime(LocalDate.of(2022, 12, 1));
//        postRollMonth.setRuleValues(new ArrayList<Integer>(){{add(2);}});
//        postRollMonth.setRollValue(16);
//        System.out.println("postRollMonth  " + checkUpdateCycle(postRollMonth, now));

        countRollMonth.setRuleType(EvalRuleTypeEnum.MONTH);
        countRollMonth.setOrderType(RollMonthOrderTypeEnum.COUNTDOWN);
        countRollMonth.setStartTime(LocalDate.of(2022, 12, 1));
        countRollMonth.setRuleValues(new ArrayList<Integer>(){{add(2);}});
        countRollMonth.setRollValue(13);
        System.out.println("countRollMonth  " + checkUpdateCycle(countRollMonth, now));


    }

    /**
     * 检查更新周期
     *
     * @return
     */
    public static Boolean checkUpdateCycle(EvalIndicator indicator, LocalDate now) {

        switch (indicator.getRuleType()) {
            case DAY: {
                //当前时间在开始时间之前，或等于开始时间，不上报数据
                if (now.isBefore(indicator.getStartTime()) || now.equals(indicator.getStartTime())) {
                    return false;
                }
                Long days = indicator.getStartTime().until(now, ChronoUnit.DAYS);
                BigDecimal remainder = new BigDecimal(days).remainder(new BigDecimal(indicator.getRollValue()));
                if (remainder.compareTo(BigDecimal.ZERO) == 0) {
                    return true;
                }
                break;
            }
            case WEEK_DAY: {
                Integer dayOfWeek = now.getDayOfWeek().getValue();
                return indicator.getRuleValues().contains(dayOfWeek);
            }
            case MONTH_FIX_DAY: {
                Integer day = now.getDayOfMonth();
                //当月是二月、且周期日期包含30号
                if (now.getMonth().getValue() == 2 && indicator.getRuleValues().contains(30)) {
                    if (now.compareTo(now.with(TemporalAdjusters.lastDayOfMonth())) == 0) {
                        return true;
                    }
                }
                return indicator.getRuleValues().contains(day);
            }
            case MONTH: {
                //当前时间在开始时间之前，或等于开始时间，不上报数据
                if (now.isBefore(indicator.getStartTime()) || now.equals(indicator.getStartTime())) {
                    return false;
                }
                Long months = indicator.getStartTime().until(now, ChronoUnit.MONTHS);
                BigDecimal remainder = new BigDecimal(months).remainder(new BigDecimal(indicator.getRuleValues().get(0)));
                if (remainder.compareTo(BigDecimal.ZERO) == 0) {
                    Integer day = indicator.getRollValue() - 1;
                    //滚动日类型；1.当月第几天 2. 当月倒数第几天
                    switch (indicator.getOrderType()) {
                        case POSITIVE : {
                            return Integer.valueOf(now.getDayOfMonth()).equals(day);
                        }
                        case COUNTDOWN : {
                            LocalDate endMonthDay = LocalDate.from(now).with(TemporalAdjusters.lastDayOfMonth());
                            System.out.println(endMonthDay.minusDays(indicator.getRollValue()));
                            return  endMonthDay.minusDays(indicator.getRollValue()).equals(now);
                        }
                    }
                }
                break;
            }
        }
        return false;
    }

    /**
     * 检查更新周期
     *
     * @return
     */
    public LocalDateTime getNextTime(EvalIndicator indicator, LocalDate now) {
        return now.plusDays(2).atTime(23, 59,59);
    }

    /**
     * 检查更新周期
     *
     * @return
     */
    public LocalDateTime getNextTime2(EvalIndicator indicator, LocalDate now) {

        LocalDate nextImportDate = now;
        switch (indicator.getRuleType()) {
            case DAY: {
                nextImportDate = now.plusDays(indicator.getRollValue());
                break;
            }
            case WEEK_DAY: {
                Integer dayOfWeek = now.getDayOfWeek().getValue();
                Integer index = indicator.getRuleValues().indexOf(dayOfWeek);
                if ((indicator.getRuleValues().size() - 1) == index) {
                    //周期（1,3,5），5到1的天数
                    nextImportDate = now.plusMonths(1).withDayOfMonth(7 - indicator.getRuleValues().get(index) + indicator.getRuleValues().get(0));
                } else {
                    nextImportDate = now.withDayOfMonth(indicator.getRuleValues().get(index + 1));
                }
                break;
            }
            case MONTH_FIX_DAY: {
                Integer dayOfMonth = now.getDayOfMonth();
                Integer index = indicator.getRuleValues().indexOf(dayOfMonth);
                Integer nextIndex = index + 1;

                if ((indicator.getRuleValues().size() - 1) == index) {
                    nextIndex = 0;
                    nextImportDate = now.plusMonths(1).withDayOfMonth(indicator.getRuleValues().get(nextIndex));
                } else {
                    nextImportDate = now.withDayOfMonth(indicator.getRuleValues().get(nextIndex));
                }
                break;
            }
            case MONTH: {
                now.plusMonths(indicator.getRuleValues().get(0));
                //滚动日类型；1.当月第几天 2. 当月倒数第几天
                switch (indicator.getOrderType()) {
                    case POSITIVE : {
                        nextImportDate = now.withDayOfMonth(indicator.getRollValue());
                        break;
                    }
                    case COUNTDOWN : {
                        LocalDate endMonthDay = LocalDate.from(now).with(TemporalAdjusters.lastDayOfMonth());
                        nextImportDate = endMonthDay.minusDays(indicator.getRollValue());
                        break;
                    }
                }
                break;
            }
        }
        return nextImportDate.minusDays(1).atTime(23, 59,59);
    }

    @Transactional(rollbackFor = Exception.class)
    public void persistenceReportTodo() {
        BoundListOperations<String, String> setOps = stringRedisTemplate.boundListOps(getEvalImportIndicator());
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        List<String> overflowsList = new ArrayList<>();
        String str;
        while ((str = setOps.leftPop()) != null) {
            final EvalGroupImportBO bo = JSONObject.parseObject(str, EvalGroupImportBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                String finalStr = str;
                threadPool.execute(() -> {
                    try {
                        saveEvalIndicatorImport(bo);
                    } catch (Exception ex) {
                        log.error("保存考评组指标录入待办失败：{}", bo, ex);
                        setOps.rightPush(finalStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsList.add(str);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsList)) {
            for (String s : overflowsList) {
                setOps.rightPushAll(s);
            }
        }
    }

    /**
     * 保存考评导入指标
     *
     * @param evalGroupImportBO
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveEvalIndicatorImport(EvalGroupImportBO evalGroupImportBO) {
        Long groupId = evalGroupImportBO.getGroupId();
        List<EvalGroup> evalGroups = evalGroupService.list(Wrappers.<EvalGroup>lambdaQuery()
                .in(EvalGroup::getId, evalGroupImportBO.getEvalGroupIds())
                .eq(EvalGroup::getYn, Boolean.TRUE)
        );

        BV.isNotEmpty(evalGroups, "考评组不存在，请重试");
        EvalIndicator evalIndicator = evalIndicatorService.queryByCode(evalGroupImportBO.getEvalIndicatorCode(), Boolean.FALSE);
        BV.notNull(evalIndicator, "考评指标不存在，请重试");

        Set<Long> allShopIds = evalGroups.stream().flatMap(evalGroup -> evalGroup.getShopIds().stream()).collect(Collectors.toSet());
        List<Long> evalGroupIds = evalGroups.stream().map(evalGroup -> evalGroup.getId()).collect(Collectors.toList());
        List<RpcUserRoleInfoDTO> userRoleInfos = this.getUserEnableRoleInfos(evalIndicator.getRoleCode(), new ArrayList<>(allShopIds));

        //没找到对应人员
        if (PublicUtil.isEmpty(userRoleInfos)) {
            return;
        }
        List<EvalIndicatorImport> indicatorImports = new ArrayList<>();
        for (RpcUserRoleInfoDTO userRoleInfo : userRoleInfos) {

            //考评组门店、管理门店取并集
            Set<Long> evalManageShopIds = userRoleInfo.getRangeList().stream()
                    .filter(range -> allShopIds.contains(range.getShopId()))
                    .map(RpcUserRoleShopDTO::getShopId).collect(Collectors.toSet());
            if (PublicUtil.isEmpty(evalManageShopIds)) {
                continue;
            }
            EvalReportVO reportVO = this.getEvalImportSnapShoot(evalGroups, evalManageShopIds, LocalDate.now());

            EvalIndicatorImport indicatorImport = this.createImport(evalIndicator, userRoleInfo, groupId);
            indicatorImport.setContent(JSON.toJSONString(reportVO));
            indicatorImport.setShopIds(new ArrayList<>(evalManageShopIds));
            indicatorImport.setEvalGroupIds(evalGroupIds);
            indicatorImport.setDataDate(LocalDate.now());
            indicatorImport.setExpireTime(evalGroupImportBO.getImportExpireTime());

            indicatorImports.add(indicatorImport);
        }

        if (PublicUtil.isNotEmpty(indicatorImports)) {
            evalIndicatorImportService.saveBatch(indicatorImports);
        }
    }

    public EvalIndicatorImport createImport(EvalIndicator evalIndicator, RpcUserRoleInfoDTO userRoleInfo, Long groupId) {
        EvalIndicatorImport indicatorImport = new EvalIndicatorImport();
        indicatorImport.setRoleCode(evalIndicator.getRoleCode());
        indicatorImport.setRoleName(evalIndicator.getRoleName());
        indicatorImport.setIndicatorCode(evalIndicator.getCode());
        indicatorImport.setIndicatorName(evalIndicator.getName());
        indicatorImport.setGroupId(groupId);
        indicatorImport.setImportUserId(userRoleInfo.getUserId());
        indicatorImport.setImportUserName(userRoleInfo.getUserName());
        indicatorImport.setYn(Boolean.TRUE);
        indicatorImport.setStatus(DataImportStatusEnum.WAIT_PUSH);
        return indicatorImport;
    }

    /**
     * 对查询的人员去重
     *
     * @param roleCode
     * @param shopIds
     * @return
     */
    public List<RpcUserRoleInfoDTO> getUserEnableRoleInfos(String roleCode, List<Long> shopIds) {
        List<RpcUserRoleInfoDTO> userRoleInfos = erpRpcService.getUserEnableRoleInfos(roleCode, shopIds);
        if (PublicUtil.isEmpty(userRoleInfos)) {
            return new ArrayList<>();
        }
        Set<Long> staffIds = new HashSet<>();
        List<RpcUserRoleInfoDTO> repeatUserRoleInfos = Lists.newArrayListWithCapacity(userRoleInfos.size());
        for (RpcUserRoleInfoDTO rpcUserRoleInfoDTO : userRoleInfos) {
            if (staffIds.add(rpcUserRoleInfoDTO.getUserId())) {
                repeatUserRoleInfos.add(rpcUserRoleInfoDTO);
            }
        }
        return repeatUserRoleInfos;
    }

    /**
     * 推送待办
     */
    @Transactional(rollbackFor = Exception.class)
    public void pushReportTodo() {
        Lock lock = distributedLocker.lock(getPushImportLockKey());
        if (! ((RLock) lock).isLocked()) {
            return;
        }
        try {
            List<EvalIndicatorImport> evalIndicatorImports = evalIndicatorImportService.list(Wrappers.<EvalIndicatorImport>lambdaQuery()
                    .eq(EvalIndicatorImport::getStatus, DataImportStatusEnum.WAIT_PUSH)
                    .eq(EvalIndicatorImport::getYn, Boolean.TRUE)
                    .last(" limit 100 ")
            );
            if (PublicUtil.isEmpty(evalIndicatorImports)) {
                return;
            }
            sendIndicatorImportTodo(evalIndicatorImports);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            lock.unlock();
        }
    }

    /**
     * 发送绩效组变动通知
     *
     * @param
     */
    @Transactional(rollbackFor = Exception.class)
    public void sendIndicatorImportTodo(List<EvalIndicatorImport> evalIndicatorImports) {
        LocalDate beginTime = LocalDate.now();
        List<BacklogBatchPlanItemReqDTO> itemList = new ArrayList<>();

        for (EvalIndicatorImport evalIndicatorImport : evalIndicatorImports) {
            Date expireTime = DateUtil.localDateTime2Date(evalIndicatorImport.getExpireTime());
            BacklogBatchPlanItemReqDTO planItemReq = new BacklogBatchPlanItemReqDTO();
            planItemReq.setDataId(evalIndicatorImport.getId().toString());
            planItemReq.setUserId(evalIndicatorImport.getImportUserId());
            planItemReq.setPlanTime(new Date());
            planItemReq.setExpireTime(expireTime);
            Map<String, String> dynamicMap = new HashMap<>(8);
            dynamicMap.put("indicatorName", evalIndicatorImport.getIndicatorName());
            dynamicMap.put("updateTime", beginTime.toString());
            dynamicMap.put("roleName", evalIndicatorImport.getRoleName());
            planItemReq.setDynamicMap(dynamicMap);

            Map<String, Object> extraData = new HashMap<>();
            extraData.put("evalIndicatorImportId", evalIndicatorImport.getId());
            planItemReq.setExtraData(extraData);
            itemList.add(planItemReq);
        }

        //推送待办
        BacklogBatchPlanReqDTO batchPlanReq = new BacklogBatchPlanReqDTO(todoVal.getEvalIndicatorImport(), itemList);
        List<FailBacklogItem> items = Optional.ofNullable(todoRpcService.batchPush(batchPlanReq)).orElse(new ArrayList<>());
        if (PublicUtil.isEmpty(items)) {
            evalIndicatorImportService.update(Wrappers.<EvalIndicatorImport>lambdaUpdate()
                    .set(EvalIndicatorImport::getStatus, DataImportStatusEnum.IMPORTING)
                    .in(EvalIndicatorImport::getId, evalIndicatorImports.stream()
                            .map(EvalIndicatorImport::getId).collect(Collectors.toList()))
                    .eq(EvalIndicatorImport::getYn, Boolean.TRUE)
            );
            return;
        }

        //更新部分数据
        log.error("发送考评数据录入待办失败，失败人员信息：{}", JSON.toJSONString(items));
        Set<Long> failIds = items.stream().map(item -> Long.parseLong(item.getDataId())).collect(Collectors.toSet());
        Set<Long> successPushIds = evalIndicatorImports.stream()
                .filter(indicatorImport -> ! failIds.contains(indicatorImport.getId()))
                .map(EvalIndicatorImport::getId).collect(Collectors.toSet());
        if (PublicUtil.isNotEmpty(successPushIds)) {
            evalIndicatorImportService.update(Wrappers.<EvalIndicatorImport>lambdaUpdate()
                    .set(EvalIndicatorImport::getStatus, DataImportStatusEnum.IMPORTING)
                    .in(EvalIndicatorImport::getId, successPushIds)
                    .eq(EvalIndicatorImport::getYn, Boolean.TRUE)
            );
        }
    }

    /**
     * 考评数据录入详情
     *
     * @return
     */
    public EvalReportVO getEvalImportSnapShoot(List<EvalGroup> evalGroups, Set<Long> manageShopIds, LocalDate dataDate) {
        EvalReportVO reportVO = new EvalReportVO();
        Map<EvalScopeEnum, List<EvalGroup>> evalScopeEnumMap = evalGroups.stream().collect(Collectors.groupingBy(EvalGroup::getScopeType));

        //人员
        List<EvalGroup> staffGroups = evalScopeEnumMap.get(EvalScopeEnum.STAFF);
        if (PublicUtil.isNotEmpty(staffGroups)) {
            List<EvalIndicatorImportUserVO> importUsers = new ArrayList<>();
            Map<Long, List<EvalGroup>> postEvalGroupMap = staffGroups.stream().collect(Collectors.groupingBy(EvalGroup::getPostId));
            for (Map.Entry<Long, List<EvalGroup>> entry : postEvalGroupMap.entrySet()) {
                Long postId = entry.getKey();
                List<Long> shopIds = entry.getValue().stream()
                                    .flatMap(evalGroup -> evalGroup.getShopIds().stream())
                                    .filter(shopId -> manageShopIds.contains(shopId))
                                    .distinct().collect(Collectors.toList());
                if (PublicUtil.isNotEmpty(shopIds)) {
                    //过滤不参加考评员工
                    List<EvalGroupUser> users = evalGroupUserBizService.fetchBuildImportUser(postId, shopIds, dataDate)
                            .stream().filter(evalGroupUser -> ! evalGroupUser.getIgnored()).collect(Collectors.toList());
                    if (PublicUtil.isNotEmpty(users)) {
                        importUsers.addAll(PublicUtil.copyList(users, EvalIndicatorImportUserVO.class));
                    }
                }
            }
            reportVO.setUsers(importUsers);
        }

        //门店
        List<EvalGroup> shopGroups = evalScopeEnumMap.get(EvalScopeEnum.SHOP);
        if (PublicUtil.isNotEmpty(shopGroups)) {
            List<Long> shopGroupIds = shopGroups.stream().flatMap(evalGroup -> evalGroup.getShopIds().stream())
                    .filter(shopId -> manageShopIds.contains(shopId))
                    .distinct().collect(Collectors.toList());
            if (PublicUtil.isNotEmpty(shopGroupIds)) {
                List<ShopDTO> shops = oopRpcService.queryShops(shopGroupIds);
                List<EvalShopVO> evalShops = new ArrayList<>();
                for (ShopDTO shop : shops) {
                    evalShops.add(new EvalShopVO(shop.getId(), shop.getShortName()));
                }
                reportVO.setShops(evalShops);
            }
        }

        return reportVO;
    }

    /**
     * 每半小时检查待办完成
     */
    @Transactional(rollbackFor = Exception.class)
    public void checkTodoCompletion() {
        Lock lock = distributedLocker.lock(getCheckTodoLockKey());
        if (((RLock) lock).isLocked()) {
            try {
                List<EvalIndicatorImport> evalIndicatorImports = evalIndicatorImportService.list(Wrappers.<EvalIndicatorImport>lambdaQuery()
                        .eq(EvalIndicatorImport::getStatus, DataImportStatusEnum.IMPORTING)
                        .eq(EvalIndicatorImport::getYn, Boolean.TRUE)
                        .last(" limit 100 ")
                );
                if (PublicUtil.isEmpty(evalIndicatorImports)) {
                    return;
                }

                for (EvalIndicatorImport indicatorImport : evalIndicatorImports) {
                    //待办是否完成
                    Boolean result = checkDataReport(indicatorImport);
                    if (result) {
                        BackLogItemDTO dto = new BackLogItemDTO(indicatorImport.getImportUserId(), getEvalImportIndicator(),
                                String.valueOf(indicatorImport.getId()), new Date(), null);
                        boolean complete = todoRpcService.cancel(dto);
                        if (complete) {
                            indicatorImport.setStatus(DataImportStatusEnum.COMPLETION);
                            evalIndicatorImportService.updateById(indicatorImport);
                        } {
                            log.error("取消考评指标导入待办失败， 导入记录：{}", JSON.toJSONString(indicatorImport));
                        }
                    }

                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                lock.unlock();
            }
        }
    }

    /**
     * 考评数据录入详情
     *
     * @param
     * @return
     */
    public Boolean checkDataReport(EvalIndicatorImport evalIndicatorImport) {

        List<Long> shopIds = evalIndicatorImport.getShopIds();
        //门店信息
        if (PublicUtil.isNotEmpty(shopIds)) {
            List<EvalIndicatorValue> evalIndicatorValues = evalIndicatorValueService.list(Wrappers.<EvalIndicatorValue>lambdaQuery()
                    .in(EvalIndicatorValue::getShopId, shopIds)
                    .eq(EvalIndicatorValue::getDimensionType, DimensionTypeEnum.SHOP)
                    .eq(EvalIndicatorValue::getDataDate, evalIndicatorImport.getDataDate())
                    .eq(EvalIndicatorValue::getYn, Boolean.TRUE)
            );
            Set<Long> reportDataShopIds = evalIndicatorValues.stream().map(EvalIndicatorValue::getShopId).collect(Collectors.toSet());

            List<Long> restShopIds = shopIds.stream()
                    .filter(shopId -> ! reportDataShopIds.contains(shopId)).collect(Collectors.toList());
            if (PublicUtil.isNotEmpty(restShopIds)) {
                log.info("考评指标导入待办，导入记录id：{}，导入门店：{}，剩余未未导入门店：{}",
                        evalIndicatorImport.getId(), JSON.toJSONString(shopIds), JSON.toJSONString(reportDataShopIds));
                return Boolean.FALSE;
            }
            return Boolean.TRUE;
        }

        //人员信息
        List<EvalGroupUser> evalGroupUsers = evalGroupUserService.list(Wrappers.<EvalGroupUser>lambdaQuery()
//                .in(EvalGroupUser::getEgc, evalIndicatorImport.getEgcs())
                .eq(EvalGroupUser::getDataDate, evalIndicatorImport.getDataDate())
                .eq(EvalGroupUser::getYn, Boolean.TRUE));
        List<Long> userIds = evalGroupUsers.stream().map(EvalGroupUser::getUserId).distinct().collect(Collectors.toList());

        List<EvalIndicatorValue> evalIndicatorValues = evalIndicatorValueService.list(Wrappers.<EvalIndicatorValue>lambdaQuery()
                .in(EvalIndicatorValue::getUserId, userIds)
                .eq(EvalIndicatorValue::getDataDate, evalIndicatorImport.getDataDate())
                .eq(EvalIndicatorValue::getDimensionType, DimensionTypeEnum.STAFF)
                .eq(EvalIndicatorValue::getYn, Boolean.TRUE)
        );

        Set<Long> reportDataUserIds = evalIndicatorValues.stream().map(EvalIndicatorValue::getUserId).collect(Collectors.toSet());

        List<Long> restUserIds = userIds.stream()
                .filter(userId -> ! reportDataUserIds.contains(userId)).collect(Collectors.toList());
        if (PublicUtil.isNotEmpty(restUserIds)) {
            log.info("考评指标导入待办，导入记录id：{}，导入门店：{}，剩余未未导入门店：{}",
                    evalIndicatorImport.getId(), JSON.toJSONString(userIds), JSON.toJSONString(restUserIds));
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    public void saveEvalReportValue(LocalDate dataDate) {
//        List<EvalIndicatorImportDetail> shopDetails =  evalIndicatorImportDetailService.getLastShopValue(dataDate);
//        List<EvalIndicatorImportDetail> userDetails =  evalIndicatorImportDetailService.getLastUserValue(dataDate);
//
//        List<EvalIndicatorValue> evalIndicatorValues = Lists.newArrayListWithCapacity(shopDetails.size() + userDetails.size());
//
//        for (EvalIndicatorImportDetail shopDetail : shopDetails) {
//
//            EvalIndicatorValue indicatorValue = new EvalIndicatorValue();
//            indicatorValue.setShopId(shopDetail.getShopId());
//            indicatorValue.setShopName(shopDetail.getShopName());
//            indicatorValue.setDimensionType(DimensionTypeEnum.SHOP);
//            indicatorValue.setIndicatorCode(shopDetail.getIndicatorCode());
//            indicatorValue.setIndicatorName(shopDetail.getIndicatorName());
//            indicatorValue.setIndicatorValue(shopDetail.getIndicatorValue());
//            indicatorValue.setValueType(shopDetail.getValueType());
//            indicatorValue.setDataDate(dataDate);
//            indicatorValue.setGroupId(shopDetail.getGroupId());
//            indicatorValue.setYn(Boolean.TRUE);
//        }
//
//        for (EvalIndicatorImportDetail shopDetail : userDetails) {
//
//            EvalIndicatorValue indicatorValue = new EvalIndicatorValue();
//            indicatorValue.setShopId(shopDetail.getShopId());
//            indicatorValue.setShopName(shopDetail.getShopName());
//            indicatorValue.setDimensionType(DimensionTypeEnum.STAFF);
//            indicatorValue.setIndicatorCode(shopDetail.getIndicatorCode());
//            indicatorValue.setIndicatorName(shopDetail.getIndicatorName());
//            indicatorValue.setIndicatorValue(shopDetail.getIndicatorValue());
//            indicatorValue.setValueType(shopDetail.getValueType());
//            indicatorValue.setDataDate(dataDate);
//            indicatorValue.setGroupId(shopDetail.getGroupId());
//            indicatorValue.setYn(Boolean.TRUE);
//        }
//
//        if (PublicUtil.isNotEmpty(evalIndicatorValues)) {
//            evalIndicatorValueService.saveBatch(evalIndicatorValues);
//        }
    }

    /**
     * 上报详情
     *
     * @param evalIndicatorImportId
     * @return
     */
    public EvalReportVO reportDetail(Long evalIndicatorImportId) {
        EvalIndicatorImport evalIndicatorImport = evalIndicatorImportService.getById(evalIndicatorImportId);
        BV.notNull(evalIndicatorImport, "导入记录不存在，请重试");
        EvalReportVO evalReportVO = new EvalReportVO();
        try {
            evalReportVO = JSON.parseObject(evalIndicatorImport.getContent(), EvalReportVO.class);
        } catch (Exception e) {
            log.error("[{}]导入数据解析失败", JSON.toJSONString(evalIndicatorImport), e);
        }

        List<EvalIndicatorImportUserVO> users = evalReportVO.getUsers();
        List<EvalShopVO> shops = evalReportVO.getShops();
        if (PublicUtil.isNotEmpty(users)) {
            List<Long> userIds = users.stream().map(EvalIndicatorImportUserVO::getUserId).collect(Collectors.toList());
            List<EvalIndicatorValue> evalIndicatorValues = evalIndicatorValueService.list(Wrappers.<EvalIndicatorValue>lambdaQuery()
                    .in(EvalIndicatorValue::getUserId, userIds)
                    .eq(EvalIndicatorValue::getDataDate, evalIndicatorImport.getDataDate())
                    .eq(EvalIndicatorValue::getDimensionType, DimensionTypeEnum.STAFF)
                    .eq(EvalIndicatorValue::getYn, Boolean.TRUE)
            );
            Set<Long> reportDataUserIds = evalIndicatorValues.stream().map(EvalIndicatorValue::getUserId).collect(Collectors.toSet());
            for (EvalIndicatorImportUserVO user : users) {
                user.setReport(reportDataUserIds.contains(user.getUserId()));
            }
        }

        if (PublicUtil.isNotEmpty(shops)) {
            List<Long> shopIds = shops.stream().map(EvalShopVO::getShopId).collect(Collectors.toList());
            List<EvalIndicatorValue> evalIndicatorValues = evalIndicatorValueService.list(Wrappers.<EvalIndicatorValue>lambdaQuery()
                    .in(EvalIndicatorValue::getShopId, shopIds)
                    .eq(EvalIndicatorValue::getDataDate, evalIndicatorImport.getDataDate())
                    .eq(EvalIndicatorValue::getDimensionType, DimensionTypeEnum.SHOP)
                    .eq(EvalIndicatorValue::getYn, Boolean.TRUE)
            );
            Set<Long> reportDataShopIds = evalIndicatorValues.stream().map(EvalIndicatorValue::getShopId).collect(Collectors.toSet());
            for (EvalShopVO shop : shops) {
                shop.setReport(reportDataShopIds.contains(shop.getShopId()));
            }
        }
        return evalReportVO;
    }
}
