package cn.fw.morax.service.biz.eval;

import cn.fw.common.page.AppPage;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.StringUtils;
import cn.fw.morax.domain.db.eval.EvalGroup;
import cn.fw.morax.domain.dto.query.EvalPoolQueryDTO;
import cn.fw.morax.domain.vo.eval.EvalShopPoolVO;
import cn.fw.morax.domain.vo.eval.EvalUserPoolVO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.erp.ErpRpcService;
import cn.fw.morax.service.biz.CommonService;
import cn.fw.morax.service.data.eval.EvalGroupService;
import cn.fw.morax.service.data.eval.EvalGroupUserService;
import cn.fw.morax.service.data.eval.EvalShopPoolService;
import cn.fw.morax.service.data.eval.EvalUserPoolService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.YearMonth;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author jiangchao
 * @des:
 * @date 2023/2/24 9:04
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class EvalPoolService {

    private final EvalGroupUserService evalGroupUserService;
    private final EvalGroupService evalGroupService;
    private final EvalUserPoolService evalUserPoolService;
    private final EvalShopPoolService evalShopPoolService;
    private final CommonService commonService;
    private final ErpRpcService erpRpcService;
    private final EhrRpcService ehrRpcService;


    /**
     * 获取绩效池
     *
     * @param dto
     * @return
     */
    public AppPage<EvalUserPoolVO> queryUserPool(EvalPoolQueryDTO dto) {
        if (PublicUtil.isNotEmpty(dto.getManagerUserId())) {
            dto.setManagerStaffIds(commonService.queryManageStaffIds(dto.getManagerUserId(), dto.getMonthly()));
        } else {
            dto.setManagerStaffIds(commonService.queryManageStaffIds(dto.getCurrentUserId(), dto.getMonthly()));
        }
        if (PublicUtil.isEmpty(dto.getManagerStaffIds())) {
            return AppPage.empty(dto);
        }

        prepareParams(dto);
        AppPage<EvalUserPoolVO> page = AppPage.empty(dto);
        log.info("绩效池查询参数：{}", JSON.toJSONString(dto));
        long total = evalUserPoolService.queryPagePoolTotal(dto);
        if (total > 0) {
            page.setTotal(total);
            List<EvalUserPoolVO> userPools = evalUserPoolService.queryPagePool(dto);
            List<EvalGroup> evalGroups = evalGroupService.list(Wrappers.<EvalGroup>lambdaQuery()
                    .in(EvalGroup::getId, userPools.stream().map(EvalUserPoolVO::getEvalGroupId).collect(Collectors.toSet()))
                    .eq(EvalGroup::getYn, Boolean.TRUE)
            );
            Map<Long, String> evalGroupNameMap = evalGroups.stream().collect(Collectors.toMap(EvalGroup::getId, EvalGroup::getName));
            for (EvalUserPoolVO userPool : userPools) {
                userPool.setEvalGroupName(evalGroupNameMap.getOrDefault(userPool.getEvalGroupId(), ""));
                userPool.convertScoreRatio();
            }
            page.setData(userPools);
        }
        return page;
    }

    /**
     * 获取绩效池
     *
     * @param dto
     * @return
     */
    public AppPage<EvalShopPoolVO> queryShopPool(EvalPoolQueryDTO dto) {
        if (PublicUtil.isNotEmpty(dto.getManagerUserId())) {
            dto.setManagerShopIds(erpRpcService.manageShopIds(dto.getManagerUserId()));
        } else {
            dto.setManagerShopIds(erpRpcService.manageShopIds(dto.getCurrentUserId()));
        }
        if (PublicUtil.isNotEmpty(dto.getShopId())) {
            if (dto.getManagerShopIds().contains(dto.getShopId())) {
                dto.setManagerShopIds(new ArrayList<Long>(){{add(dto.getShopId());}});
            } else {
                dto.setManagerShopIds(new ArrayList<Long>());
            }
        }
        if (PublicUtil.isEmpty(dto.getManagerShopIds())) {
            return AppPage.empty(dto);
        }

        prepareParams(dto);
        AppPage<EvalShopPoolVO> page = AppPage.empty(dto);
        log.info("考评池查询参数：{}", JSON.toJSONString(dto));
        long total = evalShopPoolService.queryPagePoolTotal(dto);
        if (total > 0) {
            page.setTotal(total);
            List<EvalShopPoolVO> shopPools = evalShopPoolService.queryPagePool(dto);
            List<EvalGroup> evalGroups = evalGroupService.list(Wrappers.<EvalGroup>lambdaQuery()
                    .in(EvalGroup::getId, shopPools.stream().map(EvalShopPoolVO::getEvalGroupId).collect(Collectors.toSet()))
                    .eq(EvalGroup::getYn, Boolean.TRUE)
            );
            Map<Long, String> evalGroupNameMap = evalGroups.stream().collect(Collectors.toMap(EvalGroup::getId, EvalGroup::getName));
            for (EvalShopPoolVO shopPool : shopPools) {
                shopPool.setEvalGroupName(evalGroupNameMap.getOrDefault(shopPool.getEvalGroupId(), ""));
                shopPool.convertScoreRatio();
            }
            page.setData(shopPools);
        }
        return page;
    }



    /**
     * 参数赋值
     *
     * @param queryVO
     */
    private void prepareParams(EvalPoolQueryDTO queryVO) {
        if (Objects.isNull(queryVO.getMonthly())) {
            queryVO.setMonthly(YearMonth.now());
        }
        if (Objects.nonNull(queryVO.getOrder()) && StringUtils.isValid(queryVO.getOrderAtt())) {
            StringBuilder sb = new StringBuilder(" order by ");
            sb.append(StringUtils.toColumnName(queryVO.getOrderAtt()).toLowerCase());
            if (queryVO.getOrder() == 1) {
                sb.append(" asc ");
            } else {
                sb.append(" desc ");
            }
            queryVO.setOrderString(sb.toString());
        }
        if (PublicUtil.isNotEmpty(queryVO.getQueryStaffs())) {
            List<Long> queryStaffIds = Arrays.stream(queryVO.getQueryStaffs().split(",")).map(Long::parseLong).collect(Collectors.toList());
            queryVO.getManagerStaffIds().retainAll(queryStaffIds);
        }
    }


}
