package cn.fw.morax.service.component;

import cn.fw.backlog.sdk.api.result.FailBacklogItem;
import cn.fw.morax.common.pojo.event.KpiGroupChangeEvent;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.rpc.backlog.TodoRpcService;
import cn.fw.morax.rpc.backlog.dto.BacklogBatchPlanItemReqDTO;
import cn.fw.morax.rpc.backlog.dto.BacklogBatchPlanReqDTO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.google.common.eventbus.Subscribe;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
@Slf4j
@RequiredArgsConstructor
public class KpiGroupChangeObserver {

    private final EhrRpcService ehrRpcService;
    private final TodoRpcService todoRpcService;

    @Value("${todo.kpi-group-change}")
    @Getter
    private String kpiGroupChangeTodoCode;


    /**
     * 绩效组变动通知
     *
     * @param event
     */
    @Subscribe
    public void sendKpiGroupChangeNotice(KpiGroupChangeEvent event) {
        //睡10秒保证数据落库
        try {
            Thread.sleep(10 * 1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        log.info("绩效组变动通知员工：[{}]", event);
        Long postId = event.getPostId();
        List<Long> shopIds = event.getShopIds();
        List<PerformanceStaffDTO> staffs = ehrRpcService.queryKpiStaff(postId, shopIds);
        if (PublicUtil.isEmpty(staffs)) {
            log.info("门店：{}，岗位：{}，暂无在职员工，不推送绩效组变动通知待办");
            return;
        }
        this.sendKpiGroupChangeTodo(event, staffs);
    }

    /**
     * 发送绩效组变动通知
     *
     * @param dto
     */
    private void sendKpiGroupChangeTodo(KpiGroupChangeEvent dto, List<PerformanceStaffDTO> staffs) {
        try {
            Long kpiGroupId = dto.getId();
            String postName = dto.getPostName();
            LocalDateTime now = LocalDateTime.now();
            Date expireTime = DateUtil.localDateTime2Date(now.plusDays(1));
            LocalDate beginTime = dto.getBeginTime();
            String indicatorNum = dto.getIndicatorNum().toString();
            List<BacklogBatchPlanItemReqDTO> itemList = Lists.newArrayListWithCapacity(staffs.size());
            for (PerformanceStaffDTO staffDTO : staffs) {
                Long userId = staffDTO.getId();

                BacklogBatchPlanItemReqDTO planItemReq = new BacklogBatchPlanItemReqDTO();
                planItemReq.setUserId(userId);
                planItemReq.setDataId(kpiGroupId.toString());
                planItemReq.setPlanTime(new Date());
                planItemReq.setExpireTime(expireTime);
                planItemReq.setShopId(staffDTO.getShopId());
                Map<String, String> dynamicMap = new HashMap<>(8);
                dynamicMap.put("postName", postName);
                dynamicMap.put("beginTime", beginTime.toString());
                dynamicMap.put("indicatorNum", indicatorNum);
                dynamicMap.put("reason", PublicUtil.isNotEmpty(dto.getReason()) ? dto.getReason() : "");
                planItemReq.setDynamicMap(dynamicMap);

                Map<String, Object> extraData = new HashMap<>();
                extraData.put("kpiGroupId", kpiGroupId);
                planItemReq.setExtraData(extraData);
                itemList.add(planItemReq);
            }
            BacklogBatchPlanReqDTO batchPlanReq = new BacklogBatchPlanReqDTO(getKpiGroupChangeTodoCode(), itemList);
            List<FailBacklogItem> items = todoRpcService.batchPush(batchPlanReq);
            if (PublicUtil.isNotEmpty(items)) {
                log.error("发送绩效组变动通知失败，失败人员信息：{}", JSON.toJSONString(items));
            }
        } catch (Exception e) {
            log.error("发送绩效组变动通知，", e);
        }
    }
}