package cn.fw.morax.service.data.kpi.impl;

import cn.fw.morax.dao.kpi.KpiGroupIndicatorHitLogDao;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorHitLog;
import cn.fw.morax.service.data.kpi.KpiGroupIndicatorHitLogService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 指标命中记录
 *
 * @author : kurisu
 * @version : 2.0
 * @className : KpiGroupIndicatorHitLogServiceImpl
 * @description : 指标命中记录
 * @date : 2022-12-15 10:24
 */
@Service
@Slf4j
public class KpiGroupIndicatorHitLogServiceImpl extends ServiceImpl<KpiGroupIndicatorHitLogDao, KpiGroupIndicatorHitLog> implements KpiGroupIndicatorHitLogService {
    @Override
    public void saveUnique(KpiGroupIndicatorHitLog kpiIndicatorHitLog) {
        Long kpiIndicatorId = kpiIndicatorHitLog.getKpiGroupIndicatorId();
        LocalDate dataDate = kpiIndicatorHitLog.getDataDate();
        Long userId = kpiIndicatorHitLog.getUserId();
        KpiGroupIndicatorHitLog hitLog = this.getOne(Wrappers.<KpiGroupIndicatorHitLog>lambdaQuery()
                .eq(KpiGroupIndicatorHitLog::getKpiGroupIndicatorId, kpiIndicatorId)
                .eq(KpiGroupIndicatorHitLog::getDataDate, dataDate)
                .eq(KpiGroupIndicatorHitLog::getUserId, userId)
                .eq(KpiGroupIndicatorHitLog::getYn, Boolean.TRUE)
        );
        if (Objects.nonNull(hitLog)) {
            kpiIndicatorHitLog.setId(hitLog.getId());
        }
        this.saveOrUpdate(kpiIndicatorHitLog);
    }

    @Override
    public List<KpiGroupIndicatorHitLog> queryByIndicatorIds(Long userId, List<Long> groupIndicatorIds, LocalDate localDate) {
        List<KpiGroupIndicatorHitLog> list = list(Wrappers.<KpiGroupIndicatorHitLog>lambdaQuery()
                .eq(KpiGroupIndicatorHitLog::getUserId, userId)
                .eq(KpiGroupIndicatorHitLog::getDataDate, localDate)
                .eq(KpiGroupIndicatorHitLog::getYn, Boolean.TRUE)
                .in(KpiGroupIndicatorHitLog::getKpiGroupIndicatorId, groupIndicatorIds)
        );
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list;
    }

}
